# -*- coding: utf-8 -*-


from optionaldict import optionaldict
from wechatpy.client.api.base import BaseWeChatAPI
from typing import List, Optional


class WeChatOA(BaseWeChatAPI):
    """
    OA管理

    https://work.weixin.qq.com/api/doc/90000/90135/90264
    """

    def get_template_detail(self, template_id):
        """
        查询审批模板的详情
        https://work.weixin.qq.com/api/doc/90000/90135/91982

        :param template_id: 模板Id
        :return:
        """
        data = {"template_id": template_id}
        return self._post("oa/gettemplatedetail", data=data)

    def get_approval_info(self, start_time, end_time, cursor, size=100, filters=None):
        """
        批量获取审批单号
        https://work.weixin.qq.com/api/doc/90000/90135/91816

        :param start_time: 开始时间戳
        :param end_time: 结束时间戳，请求的参数endtime需要大于startime， 起始时间跨度不能超过31天；
        :param cursor: 分页查询游标，默认为0，后续使用返回的next_cursor进行分页拉取
        :param size: 一次请求拉取审批单数量，默认值为100，上限值为100
        :param filters: 请自行查看文档
        :return:
        """
        data = optionaldict(
            {"starttime": str(start_time), "endtime": str(end_time), "cursor": cursor, "size": size, "filter": filters}
        )

        return self._post("oa/getapprovalinfo", data=data)

    def get_approval_detail(self, sp_no):
        """
        获取审批申请详情
        https://work.weixin.qq.com/api/doc/90000/90135/91983

        :param sp_no: 审批单编号
        :return:
        """
        data = {"sp_no": sp_no}
        return self._post("oa/getapprovaldetail", data=data)

    def apply_event(
        self,
        creator_userid,
        template_id,
        use_template_approver,
        approver,
        apply_data,
        summary_list,
        notifyer=None,
        notify_type=None,
    ):
        """
        提交审批申请，这个函数的参数比较复杂，具体请查看官方文档
        https://work.weixin.qq.com/api/doc/90000/90135/91853

        :param creator_userid: 申请人userid，此审批申请将以此员工身份提交，申请人需在应用可见范围内
        :param template_id: 模板id。可在“获取审批申请详情”、“审批状态变化回调通知”中获得，也可在审批模板的模板编辑页面链接中获得。暂不支持通过接口提交[打卡补卡][调班]模板审批单。
        :param use_template_approver: 审批人模式：0-通过接口指定审批人、抄送人（此时approver、notifyer等参数可用）; 1-使用此模板在管理后台设置的审批流程，支持条件审批。
        :param approver: 具体参数查看官方文档，审批流程信息，用于指定审批申请的审批流程，支持单人审批、多人会签、多人或签，可能有多个审批节点，仅use_template_approver为0时生效。
        :param apply_data: 具体参数查看官方文档，审批申请数据，可定义审批申请中各个控件的值，其中必填项必须有值，选填项可为空，数据结构同“获取审批申请详情”接口返回值中同名参数“apply_data”
        :param summary_list: 具体参数查看官方文档，摘要信息，用于显示在审批通知卡片、审批列表的摘要信息，最多3行
        :param notifyer: 抄送人节点userid列表，仅use_template_approver为0时生效。
        :param notify_type: 抄送方式：1-提单时抄送（默认值）； 2-单据通过后抄送；3-提单和单据通过后抄送。仅use_template_approver为0时生效。
        :return:
        """
        data = optionaldict(
            {
                "creator_userid": creator_userid,
                "template_id": template_id,
                "use_template_approver": use_template_approver,
                "approver": approver,
                "notifyer": notifyer,
                "notify_type": notify_type,
                "apply_data": apply_data,
                "summary_list": summary_list,
            }
        )
        return self._post("oa/applyevent", data=data)

    def get_dial_record(
        self, start_time: Optional[int] = None, end_time: Optional[int] = None, offset: int = 0, limit: int = 100
    ) -> dict:
        """
        获取公费电话拨打记录
        https://work.weixin.qq.com/api/doc/90000/90135/90267

        企业可通过此接口，按时间范围拉取成功接通的公费电话拨打记录。

        请注意，查询的时间范围为[start_time,end_time]，即前后均为闭区间。在两个参数都
        指定了的情况下，结束时间不得小于开始时间，开始时间也不得早于当前时间，否则会返回
        600018错误码(无效的起止时间)。

        受限于网络传输，起止时间的最大跨度为30天，如超过30天，则以结束时间为基准向前取
        30天进行查询。

        如果未指定起止时间，则默认查询最近30天范围内数据。

        :param start_time: 查询的起始时间戳
        :param end_time: 查询的结束时间戳
        :param offset: 分页查询的偏移量
        :param limit: 分页查询的每页大小,默认为100条，如该参数大于100则按100处理
        :return: 公费电话拨打记录
        """
        if start_time and end_time and end_time <= start_time:
            raise ValueError("the end time must be greater than the beginning time")

        data = {"start_time": start_time, "end_time": end_time, "offset": offset, "limit": limit}
        return self._post("dial/get_dial_record", data=data)

    def get_checkin_data(self, data_type: int, start_time: int, end_time: int, userid_list: List[str]) -> dict:
        """
        获取打卡数据
        https://work.weixin.qq.com/api/doc/90000/90135/90262

        - 获取记录时间跨度不超过30天
        - 用户列表不超过100个。若用户超过100个，请分批获取
        - 有打卡记录即可获取打卡数据，与当前”打卡应用”是否开启无关

        :param data_type: 打卡类型。1：上下班打卡；2：外出打卡；3：全部打卡
        :param start_time: 获取打卡记录的开始时间。Unix时间戳
        :param end_time: 获取打卡记录的结束时间。Unix时间戳
        :param userid_list: 需要获取打卡记录的用户列表
        :return: 打卡数据
        """
        checkin_data_type = {1: "上下班打卡", 2: "外出打卡", 3: "全部打卡"}
        if data_type not in checkin_data_type:
            raise ValueError(f"data_type must be in {list(checkin_data_type.keys())}")

        if end_time <= start_time:
            raise ValueError("the end time must be greater than the beginning time")

        if not userid_list:
            raise ValueError("the userid_list can't be an empty list")

        data = {
            "opencheckindatatype": data_type,
            "starttime": start_time,
            "endtime": end_time,
            "useridlist": userid_list,
        }
        return self._post("checkin/getcheckindata", data=data)

    def get_checkin_option(self, datetime: int, userid_list: List[str]) -> dict:
        """
        获取打卡规则
        https://work.weixin.qq.com/api/doc/90000/90135/90263

        - 用户列表不超过100个，若用户超过100个，请分批获取。
        - 用户在不同日期的规则不一定相同，请按天获取。

        :param datetime: 需要获取规则的日期当天0点的Unix时间戳
        :param userid_list: 需要获取打卡规则的用户列表
        :return: 打卡规则
        """
        if not userid_list:
            raise ValueError("the userid_list can't be an empty list")

        data = {"datetime": datetime, "useridlist": userid_list}
        return self._post("checkin/getcheckinoption", data=data)

    def get_open_approval_data(self, third_no: str) -> dict:
        """
        查询自建应用审批单当前状态
        https://work.weixin.qq.com/api/doc/90000/90135/90269

        :param third_no: 开发者发起申请时定义的审批单号
        :return: 审批单的当前审批状态
        """
        data = {"thirdNo": third_no}
        return self._post("corp/getopenapprovaldata", data=data)



# -*- coding: utf-8 -*-


from wechatpy.client.base import BaseWeChatClient
from wechatpy.work.client import api


class WeChatClient(BaseWeChatClient):
    API_BASE_URL = "https://qyapi.weixin.qq.com/cgi-bin/"

    agent = api.WeChatAgent()
    appchat = api.WeChatAppChat()
    batch = api.WeChatBatch()
    calendar = api.WeChatCalendar()
    department = api.WeChatDepartment()
    email = api.WeChatEMail()
    external_contact = api.WeChatExternalContact()
    external_contact_group_chat = api.WeChatExternalContactGroupChat()
    jsapi = api.WeChatJSAPI()
    media = api.WeChatMedia()
    menu = api.WeChatMenu()
    message = api.WeChatMessage()
    misc = api.WeChatMisc()
    oauth = api.WeChatOAuth()
    schedule = api.WeChatSchedule()
    service = api.WeChatService()
    tag = api.WeChatTag()
    user = api.WeChatUser()
    oa = api.WeChatOA()
    invoice = api.WeChatInvoice()

    def __init__(
        self,
        corp_id,
        secret,
        access_token=None,
        session=None,
        timeout=None,
        auto_retry=True,
    ):
        self.corp_id = corp_id
        self.secret = secret
        super().__init__(corp_id, access_token, session, timeout, auto_retry)

    @property
    def access_token_key(self):
        return f"{self.corp_id}_{self.secret[:10]}_access_token"

    def fetch_access_token(self):
        """Fetch access token"""
        return self._fetch_access_token(
            url="https://qyapi.weixin.qq.com/cgi-bin/gettoken",
            params={"corpid": self.corp_id, "corpsecret": self.secret},
        )



# -*- coding: utf-8 -*-
"""
    wechatpy.client.jsapi
    ~~~~~~~~~~~~~~~~~~~~

    This module provides some APIs for JS SDK

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""


import hashlib
import time
from typing import Optional

from wechatpy.utils import WeChatSigner, random_string
from wechatpy.client.api.base import BaseWeChatAPI
from wechatpy.schemes import JsApiCardExt


class WeChatJSAPI(BaseWeChatAPI):
    def get_ticket(self, type="jsapi"):
        """
        获取微信 JS-SDK ticket

        :return: 返回的 JSON 数据包
        """
        return self._get("ticket/getticket", params={"type": type})

    def get_jsapi_ticket(self):
        """
        获取微信 JS-SDK ticket

        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        ticket_key = f"{self.appid}_jsapi_ticket"
        expires_at_key = f"{self.appid}_jsapi_ticket_expires_at"
        ticket = self.session.get(ticket_key)
        expires_at = self.session.get(expires_at_key) or 0
        if not ticket or expires_at < int(time.time()):
            jsapi_ticket_response = self.get_ticket("jsapi")
            ticket = jsapi_ticket_response["ticket"]
            expires_at = int(time.time()) + int(jsapi_ticket_response["expires_in"])
            self.session.set(ticket_key, ticket)
            self.session.set(expires_at_key, expires_at)
        return ticket

    def get_jsapi_signature(self, noncestr, ticket, timestamp, url):
        """
        获取 JSAPI 签名

        :param noncestr: nonce string
        :param ticket: JS-SDK ticket
        :param timestamp: 时间戳
        :param url: URL
        :return: 签名
        """
        data = [
            f"noncestr={noncestr}",
            f"jsapi_ticket={ticket}",
            f"timestamp={timestamp}",
            f"url={url}",
        ]
        signer = WeChatSigner(delimiter=b"&")
        signer.add_data(*data)
        return signer.signature

    def get_jsapi_card_ticket(self):
        """
        获取 api_ticket：是用于调用微信卡券JS API的临时票据, 有效期为7200 秒, 通过access_token 来获取.
        微信文档地址：https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/JS-SDK.html#62
        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        jsapi_card_ticket_key = f"{self.appid}_jsapi_card_ticket"
        jsapi_card_ticket_expire_at_key = f"{self.appid}_jsapi_card_ticket_expires_at"

        ticket = self.session.get(jsapi_card_ticket_key)
        expires_at = self.session.get(jsapi_card_ticket_expire_at_key, 0)
        if not ticket or int(expires_at) < int(time.time()):
            ticket_response = self.get_ticket("wx_card")
            ticket = ticket_response["ticket"]
            expires_at = int(time.time()) + int(ticket_response["expires_in"])
            self.session.set(jsapi_card_ticket_key, ticket)
            self.session.set(jsapi_card_ticket_expire_at_key, expires_at)
        return ticket

    def get_jsapi_add_card_params(
        self,
        card_id: str,
        code: str = "",
        openid: str = "",
        fixed_begintimestamp: Optional[int] = None,
        outer_str: str = "",
        nonce_str: str = "",
        timestamp: int = 0,
        card_ticket: str = "",
    ) -> JsApiCardExt:
        """
        用于生成 jsapi 批量添加卡券接口的 cardList 参数中的 cardExt 参数
        参数意义见微信文档地址：
            https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/JS-SDK.html#65 和
            https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/WeChat_Coupon_Interface.html#4
        :param card_id: 卡券ID. 一个卡券ID对应一类卡券, 包含了相应库存数量的Code码.
        :param code: 卡券Code码. 一张卡券的唯一标识, 核销卡券时使用此串码, 支持商户自定义.
        :param openid: 用户在该公众号下的唯一身份.
        :param fixed_begintimestamp: 卡券在第三方系统的实际领取时间, 为东八区时间戳 (UTC+8,精确到秒) . 当卡券的有效期类型为
                                     DATE_TYPE_FIX_TERM 时专用, 标识卡券的实际生效时间,
                                     用于解决商户系统内起始时间和领取时间不同步的问题.
        :param outer_str: 领取渠道参数, 用于标识本次领取的渠道值.  支持商户自定义场景值填入card_ext进行卡券投放,
                          当用户领取时会将相应场景值通过事件通知商户.
        :param nonce_str: 随机字符串, 由开发者设置传入, 加强安全性 (若不填写可能被重放请求).
        :param timestamp: unix 时间戳, 不同添加请求的时间戳须动态生成, 若重复将会导致领取失败.
        :param card_ticket: 用于卡券的微信 api_ticket

        :return: 卡券的附加信息 card_ext 的 dict
        """
        nonce_str = nonce_str or random_string()
        timestamp = timestamp or int(time.time())
        card_ticket = card_ticket or self.get_jsapi_card_ticket()

        card_signature_dict = {
            "nonce_str": nonce_str,
            "api_ticket": card_ticket,
            "timestamp": str(timestamp),
            "code": code,
            "openid": openid,
            "card_id": card_id,
        }
        list_before_sign = sorted([str(x) for x in card_signature_dict.values()])
        str_to_sign = "".join(list_before_sign).encode()
        card_ext = JsApiCardExt(
            code=code,
            openid=openid,
            timestamp=str(timestamp),
            fixed_begintimestamp=fixed_begintimestamp,
            outer_str=outer_str,
            nonce_str=nonce_str,
            signature=hashlib.sha1(str_to_sign).hexdigest(),
        )

        return card_ext



# -*- coding: utf-8 -*-


from itertools import chain
from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatDepartment(BaseWeChatAPI):
    def create(self, name, parent_id=1, order=None, id=None, name_en=None):
        """
        创建部门

        详情请参考
        https://developer.work.weixin.qq.com/document/path/90205

        :param name: 部门名称。长度限制为1~32个字符，字符不能包括\\:?”<>｜
        :param parent_id: 父部门id，32位整型
        :param order: 在父部门中的次序值。order值大的排序靠前。有效的值范围是[0, 2^32)
        :param id: 部门id，32位整型，指定时必须大于1。若不填该参数，将自动生成id
        :param name_en: 英文名称。同一个层级的部门名称不能重复。需要在管理后台开启多语言支持才能生效。长度限制为1~32个字符，字符不能包括:*?"<>｜
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(name=name, parentid=parent_id, order=order, id=id, name_en=name_en)
        return self._post("department/create", data=data)

    def update(self, id, name=None, parent_id=None, order=None, name_en=None):
        """
        更新部门

        详情请参考
        https://developer.work.weixin.qq.com/document/path/90206

        :param id: 部门 id
        :param name: 部门名称。长度限制为1~32个字符，字符不能包括\\:?”<>｜
        :param parent_id: 父亲部门id
        :param order: 在父部门中的次序值。order值大的排序靠前。有效的值范围是[0, 2^32)
        :param name_en: 英文名称。同一个层级的部门名称不能重复。需要在管理后台开启多语言支持才能生效。长度限制为1~32个字符，字符不能包括:*?"<>｜
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(id=id, name=name, parentid=parent_id, order=order, name_en=name_en)
        return self._post("department/update", data=data)

    def delete(self, id):
        """
        删除部门

        详情请参考
        https://developer.work.weixin.qq.com/document/path/90207

        :param id: 部门id。（注：不能删除根部门；不能删除含有子部门、成员的部门）
        :return: 返回的 JSON 数据包
        """
        return self._get("department/delete", params={"id": id})

    def list(self, id=None):
        """
        获取指定部门列表

        详情请参考
        https://developer.work.weixin.qq.com/document/path/90208

        权限说明：
        只能拉取token对应的应用的权限范围内的部门列表

        :param id: 部门id。获取指定部门及其下的子部门。 如果不填，默认获取全量组织架构
        :return: 部门列表
        """
        if id is None:
            res = self._get("department/list")
        else:
            res = self._get("department/list", params={"id": id})
        return res["department"]

    def simple_list(self, id=None):
        """
        获取子部门 ID 列表，和 list 接口相比，此接口只返回部门 ID，ORDER 和 PARENTID 字段

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95350

        :param id: 部门id。获取指定部门及其下的子部门（以及子部门的子部门等等，递归）。 如果不填，默认获取全量组织架构
        :return: 部门列表
        """
        if id is None:
            res = self._get("department/simplelist")
        else:
            res = self._get("department/simplelist", params={"id": id})
        return res["department_id"]

    def get(self, id):
        """
        获取单个部门详情

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95351

        :param id: 部门 ID
        :return: 部门信息
        """
        res = self._get("department/get", params={"id": id})
        return res["department"]

    def get_users(self, id, fetch_child=0, simple=True):
        """
        获取部门成员：https://developer.work.weixin.qq.com/document/path/90200

        获取部门成员详情：https://developer.work.weixin.qq.com/document/path/90201

        :param id: 部门 id
        :param fetch_child: 1/0：是否递归获取子部门下面的成员
        :param simple: True 获取部门成员，False 获取部门成员详情
        :return: 部门成员列表
        """
        url = "user/simplelist" if simple else "user/list"
        res = self._get(
            url,
            params={
                "department_id": id,
                "fetch_child": 1 if fetch_child else 0,
            },
        )
        return res["userlist"]

    def get_map_users(self, id=None, key="name", fetch_child=0):
        """
        映射员工某详细字段到 ``user_id``

        企业微信许多对员工操作依赖于 ``user_id`` ，但没有提供直接查询员工对应 ``user_id`` 的结构，

        这里是一个变通的方法，常用于储存员工 ``user_id`` ，并用于后续查询或对单人操作（如发送指定消息）

        :param id: 部门 id， 如果不填，默认获取有权限的所有部门
        :param key: 员工详细信息字段 key，所指向的值必须唯一
        :param fetch_child: 1/0：是否递归获取子部门下面的成员
        :return: dict - 部门成员指定字段到 user_id 的 map  ``{ key: user_id }``
        """
        ids = [id] if id is not None else [item["id"] for item in self.list()]
        users_info = list(
            chain(*[self.get_users(department, fetch_child=fetch_child, simple=False) for department in ids])
        )
        users_zip = [(user[key], user["userid"]) for user in users_info]
        return dict(users_zip)



# -*- coding: utf-8 -*-

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatSemantic(BaseWeChatAPI):
    def search(
        self,
        query,
        category,
        uid=None,
        latitude=None,
        longitude=None,
        city=None,
        region=None,
    ):
        """
        发送语义理解请求
        详情请参考
        http://mp.weixin.qq.com/wiki/0/0ce78b3c9524811fee34aba3e33f3448.html

        :param query: 输入文本串
        :param category: 需要使用的服务类型，多个可传入列表
        :param uid: 可选，用户唯一id（非开发者id），用户区分公众号下的不同用户（建议填入用户openid）
        :param latitude: 可选，纬度坐标，与经度同时传入；与城市二选一传入
        :param longitude: 可选，经度坐标，与纬度同时传入；与城市二选一传入
        :param city: 可选，城市名称，与经纬度二选一传入
        :param region: 可选，区域名称，在城市存在的情况下可省；与经纬度二选一传入
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.semantic.search(
                '查一下明天从北京到上海的南航机票',
                'flight,hotel',
                city='北京'
            )

        """
        if isinstance(category, (tuple, list)):
            category = ",".join(category)

        data = optionaldict()
        data["query"] = query
        data["category"] = category
        data["uid"] = uid
        data["latitude"] = latitude
        data["longitude"] = longitude
        data["city"] = city
        data["region"] = region
        data["appid"] = self._client.appid
        return self._post(url="https://api.weixin.qq.com/semantic/semproxy/search", data=data)



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMiniProgram(BaseWeChatAPI):
    """
    小程序接口（服务商、第三方应用开发相关）

    https://work.weixin.qq.com/api/doc/90001/90144/92423

    新的授权体系有部分接口未实现，欢迎提交 PR。
    """

    def jscode2session(self, js_code):
        """
        临时登录凭证校验接口

        详情请参考
        https://work.weixin.qq.com/api/doc/90001/90143/90603
        :param js_code: 登录时获取的 code
        :return: 返回的 JSON 数据包
        """
        return self._get(
            "service/miniprogram/jscode2session",
            params={"js_code": js_code, "grant_type": "authorization_code"},
        )



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatAuth(BaseWeChatAPI):
    """
    应用授权（服务商、第三方应用开发相关）

    https://work.weixin.qq.com/api/doc/90001/90143/90597

    新的授权体系有部分接口未实现，欢迎提交 PR。
    """

    def get_pre_auth_code(self):
        """
        获取预授权码

        详情请参考
        https://open.work.weixin.qq.com/api/doc/90001/90143/90601
        :return: 返回的 JSON 数据包
        """
        return self._get(
            "service/get_pre_auth_code",
        )

    def set_session_info(self, pre_auth_code, app_id=None, auth_type=0):
        """
        设置授权配置

        详情请参考
        https://open.work.weixin.qq.com/api/doc/90001/90143/90602
        :param pre_auth_code: 预授权码
        :param app_id: 允许进行授权的应用id，如1、2、3， 不填或者填空数组都表示允许授权套件内所有应用
                       （仅旧的多应用套件可传此参数，新开发者可忽略）
        :param auth_type: 授权类型：0 正式授权， 1 测试授权。 默认值为0。注意，请确保应用在正式发布后的授权类型为“正式授权”
        :return: 返回的 JSON 数据包
        """
        session_info = {"auth_type": auth_type}
        if app_id:
            session_info["appid"] = app_id
        return self._post(
            "service/set_session_info",
            data={"pre_auth_code": pre_auth_code, "session_info": session_info},
        )

    def get_permanent_code(self, auth_code):
        """
        获取企业永久授权码

        详情请参考
        https://work.weixin.qq.com/api/doc/90001/90143/90603
        :param auth_code: 临时授权码，会在授权成功时附加在redirect_uri中跳转回第三方服务商网站，或通过回调推送给服务商。长度为64至512个字节
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "service/get_permanent_code",
            data={
                "auth_code": auth_code,
            },
        )

    def get_auth_info(self, auth_code, permanent_code):
        """
        获取企业授权信息

        详情请参考
        https://work.weixin.qq.com/api/doc/10975
        :param auth_code: 临时授权码，会在授权成功时附加在redirect_uri中跳转回第三方服务商网站，或通过回调推送给服务商。长度为64至512个字节
        :param permanent_code: 	永久授权码，通过get_permanent_code获取
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "service/get_auth_info",
            data={
                "auth_code": auth_code,
                "permanent_code": permanent_code,
            },
        )



# -*- coding: utf-8 -*-

from datetime import datetime, date

from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatTools(BaseWeChatPayAPI):
    def short_url(self, long_url):
        """
        长链接转短链接

        :param long_url: 长链接
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "long_url": long_url,
        }
        return self._post("tools/shorturl", data=data)

    def download_bill(self, bill_date, bill_type="ALL", device_info=None):
        """
        下载对账单

        :param bill_date: 下载对账单的日期
        :param bill_type: 账单类型，ALL，返回当日所有订单信息，默认值
                          SUCCESS，返回当日成功支付的订单,
                          REFUND，返回当日退款订单,
                          REVOKED，已撤销的订单
        :param device_info: 微信支付分配的终端设备号，填写此字段，只下载该设备号的对账单
        :return: 返回的结果数据
        """
        if isinstance(bill_date, (datetime, date)):
            bill_date = bill_date.strftime("%Y%m%d")

        data = {
            "appid": self.appid,
            "bill_date": bill_date,
            "bill_type": bill_type,
            "device_info": device_info,
        }
        return self._post("pay/downloadbill", data=data)

    def download_fundflow(self, bill_date, account_type="Basic", tar_type=None):
        """
        下载资金账单
        https://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=9_18&index=7

        :param bill_date: 下载对账单的日期
        :param account_type: 账单的资金来源账户
                             Basic  基本账户
                             Operation 运营账户
                             Fees 手续费账户
        :param tar_type: 非必传参数，固定值：GZIP，返回格式为.gzip的压缩包账单。
                         不传则默认为数据流形式。
        """
        if isinstance(bill_date, (datetime, date)):
            bill_date = bill_date.strftime("%Y%m%d")

        data = {
            "appid": self.appid,
            "bill_date": bill_date,
            "account_type": account_type,
            "sign_type": "HMAC-SHA256",
        }
        if tar_type is not None:
            data["tar_type"] = tar_type
        return self._post("pay/downloadfundflow", data=data)

    def auto_code_to_openid(self, auth_code):
        """
        授权码查询 openid 接口

        :param auth_code: 扫码支付授权码，设备读取用户微信中的条码或者二维码信息
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "auth_code": auth_code,
        }
        return self._post("tools/authcodetoopenid", data=data)



# -*- coding: utf-8 -*-
import json
import time
import random
from datetime import datetime, timedelta

from wechatpy.utils import timezone
from wechatpy.pay.utils import get_external_ip
from wechatpy.pay.api.base import BaseWeChatPayAPI
from wechatpy.utils import random_string, to_text
from wechatpy.pay.utils import calculate_signature


class WeChatOrder(BaseWeChatPayAPI):
    def create(
        self,
        trade_type,
        body,
        total_fee,
        notify_url,
        client_ip=None,
        user_id=None,
        out_trade_no=None,
        detail=None,
        attach=None,
        fee_type="CNY",
        time_start=None,
        time_expire=None,
        goods_tag=None,
        product_id=None,
        device_info=None,
        limit_pay=None,
        scene_info=None,
        sub_user_id=None,
        **kwargs,
    ):
        """
        统一下单接口

        :param trade_type: 交易类型，取值如下：JSAPI，NATIVE，APP，WAP, MWEB
        :param body: 商品描述
        :param total_fee: 总金额，单位分
        :param notify_url: 接收微信支付异步通知回调地址
        :param client_ip: 可选，APP和网页支付提交用户端ip，Native支付填调用微信支付API的机器IP
        :param user_id: 可选，用户在商户appid下的唯一标识。trade_type=JSAPI和appid已设定，此参数必传
        :param sub_user_id: 可选，小程序appid下的唯一标识。trade_type=JSAPI和sub_appid已设定，此参数必传
        :param out_trade_no: 可选，商户订单号，默认自动生成
        :param detail: 可选，商品详情
        :param attach: 可选，附加数据，在查询API和支付通知中原样返回，该字段主要用于商户携带订单的自定义数据
        :param fee_type: 可选，符合ISO 4217标准的三位字母代码，默认人民币：CNY
        :param time_start: 可选，订单生成时间，默认为当前时间
        :param time_expire: 可选，订单失效时间，默认为订单生成时间后两小时
        :param goods_tag: 可选，商品标记，代金券或立减优惠功能的参数
        :param product_id: 可选，trade_type=NATIVE，此参数必传。此id为二维码中包含的商品ID，商户自行定义
        :param device_info: 可选，终端设备号(门店号或收银设备ID)，注意：PC网页或公众号内支付请传"WEB"
        :param limit_pay: 可选，指定支付方式，no_credit--指定不能使用信用卡支付
        :param scene_info: 可选，上报支付的场景信息
        :param kwargs: 其他未列举在上述参数中的统一下单接口调用参数,例如电子发票入口开放标识receipt
        :type scene_info: dict
        :return: 返回的结果数据
        """
        now = datetime.fromtimestamp(time.time(), tz=timezone("Asia/Shanghai"))
        hours_later = now + timedelta(hours=2)
        if time_start is None:
            time_start = now
        else:
            time_start = time_start.astimezone(timezone("Asia/Shanghai"))
        if time_expire is None:
            time_expire = hours_later
        else:
            time_expire = time_expire.astimezone(timezone("Asia/Shanghai"))
        if not out_trade_no:
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        if scene_info is not None:
            scene_info = json.dumps(scene_info, ensure_ascii=False)
        data = {
            "appid": self.appid,
            "sub_appid": self.sub_appid,
            "device_info": device_info,
            "body": body,
            "detail": detail,
            "attach": attach,
            "out_trade_no": out_trade_no,
            "fee_type": fee_type,
            "total_fee": total_fee,
            "spbill_create_ip": client_ip or get_external_ip(),
            "time_start": time_start.strftime("%Y%m%d%H%M%S"),
            "time_expire": time_expire.strftime("%Y%m%d%H%M%S"),
            "goods_tag": goods_tag,
            "notify_url": notify_url,
            "trade_type": trade_type,
            "limit_pay": limit_pay,
            "product_id": product_id,
            "openid": user_id,
            "sub_openid": sub_user_id,
            "scene_info": scene_info,
        }
        data.update(kwargs)
        return self._post("pay/unifiedorder", data=data)

    def query(self, transaction_id=None, out_trade_no=None):
        """
        查询订单

        :param transaction_id: 微信的订单号，优先使用
        :param out_trade_no: 商户系统内部的订单号，当没提供transaction_id时需要传这个。
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
        }
        return self._post("pay/orderquery", data=data)

    def close(self, out_trade_no):
        """
        关闭订单

        :param out_trade_no: 商户系统内部的订单号
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "out_trade_no": out_trade_no,
        }
        return self._post("pay/closeorder", data=data)

    def get_appapi_params(self, prepay_id, timestamp=None, nonce_str=None):
        """
        获取 APP 支付参数

        :param prepay_id: 统一下单接口返回的 prepay_id 参数值
        :param timestamp: 可选，时间戳，默认为当前时间戳
        :param nonce_str: 可选，随机字符串，默认自动生成
        :return: 签名
        """
        data = {
            "appid": self.appid,
            "partnerid": self.mch_id,
            "prepayid": prepay_id,
            "package": "Sign=WXPay",
            "timestamp": timestamp or to_text(int(time.time())),
            "noncestr": nonce_str or random_string(32),
        }
        sign = calculate_signature(data, self._client.api_key)
        data["sign"] = sign
        return data

    def reverse(self, transaction_id=None, out_trade_no=None):
        """
        撤销订单

        :param transaction_id: 可选，微信的订单号，优先使用
        :param out_trade_no: 可选，商户系统内部的订单号,
                            transaction_id、out_trade_no二选一，
                            如果同时存在优先级：transaction_id> out_trade_no
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
        }
        return self._post("secapi/pay/reverse", data=data)



# -*- coding: utf-8 -*-

import random
from datetime import datetime

from wechatpy.pay.utils import get_external_ip
from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatRedpack(BaseWeChatPayAPI):
    def send(
        self,
        user_id,
        total_amount,
        send_name,
        act_name,
        wishing,
        remark,
        total_num=1,
        client_ip=None,
        out_trade_no=None,
        scene_id=None,
        consume_mch_id=None,
    ):
        """
        发送现金红包

        :param user_id: 接收红包的用户在公众号下的 openid
        :param total_amount: 红包金额，单位分
        :param send_name: 商户名称
        :param act_name: 活动名称
        :param wishing: 红包祝福语
        :param remark: 备注
        :param client_ip: 可选，调用接口的机器 IP 地址
        :param total_num: 可选，红包发放总人数，默认为 1
        :param out_trade_no: 可选，商户订单号，默认会自动生成
        :param scene_id: 可选，发放红包使用场景，红包金额大于200时必传
        :param consume_mch_id: 可选，资金授权商户号。服务商替特约商户发放时使用
        :return: 返回的结果数据字典
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "wxappid": self.appid,
            "re_openid": user_id,
            "total_amount": total_amount,
            "send_name": send_name,
            "act_name": act_name,
            "wishing": wishing,
            "remark": remark,
            "client_ip": client_ip or get_external_ip(),
            "total_num": total_num,
            "mch_billno": out_trade_no,
            "scene_id": scene_id,
            "risk_info": None,
            "consume_mch_id": consume_mch_id,
        }
        return self._post("mmpaymkttransfers/sendredpack", data=data)

    def send_group(
        self,
        user_id,
        total_amount,
        send_name,
        act_name,
        wishing,
        remark,
        total_num,
        client_ip=None,
        amt_type="ALL_RAND",
        out_trade_no=None,
        scene_id=None,
        consume_mch_id=None,
    ):
        """
        发送裂变红包

        :param user_id: 接收红包的用户在公众号下的 openid
        :param total_amount: 红包金额，单位分
        :param send_name: 商户名称
        :param act_name: 活动名称
        :param wishing: 红包祝福语
        :param remark: 备注
        :param total_num: 红包发放总人数
        :param client_ip: 可选，调用接口的机器 IP 地址
        :param amt_type: 可选，红包金额设置方式
                         ALL_RAND—全部随机,商户指定总金额和红包发放总人数，由微信支付随机计算出各红包金额
        :param out_trade_no: 可选，商户订单号，默认会自动生成
        :param scene_id: 可选，发放红包使用场景，红包金额大于200时必传
        :param consume_mch_id: 可选，资金授权商户号。服务商替特约商户发放时使用
        :return: 返回的结果数据字典
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = f"{self._client.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "wxappid": self.appid,
            "re_openid": user_id,
            "total_amount": total_amount,
            "send_name": send_name,
            "act_name": act_name,
            "wishing": wishing,
            "remark": remark,
            "total_num": total_num,
            "client_ip": client_ip or get_external_ip(),
            "amt_type": amt_type,
            "mch_billno": out_trade_no,
            "scene_id": scene_id,
            "risk_info": None,
            "consume_mch_id": consume_mch_id,
        }
        return self._post("mmpaymkttransfers/sendgroupredpack", data=data)

    def query(self, out_trade_no, bill_type="MCHT"):
        """
        查询红包发放记录

        :param out_trade_no: 商户订单号
        :param bill_type: 可选，订单类型，目前固定为 MCHT
        :return: 返回的红包发放记录信息
        """
        data = {
            "mch_billno": out_trade_no,
            "bill_type": bill_type,
            "appid": self.appid,
        }
        return self._post("mmpaymkttransfers/gethbinfo", data=data)



# -*- coding: utf-8 -*-

import random
from datetime import datetime

from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatCoupon(BaseWeChatPayAPI):
    def send(self, user_id, stock_id, op_user_id=None, device_info=None, out_trade_no=None):
        """
        发放代金券

        :param user_id: 用户在公众号下的 openid
        :param stock_id: 代金券批次 ID
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :param out_trade_no: 可选，商户订单号，需保持唯一性，默认自动生成
        :return: 返回的结果信息
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "appid": self.appid,
            "coupon_stock_id": stock_id,
            "openid": user_id,
            "openid_count": 1,
            "partner_trade_no": out_trade_no,
            "op_user_id": op_user_id,
            "device_info": device_info,
            "version": "1.0",
            "type": "XML",
        }
        return self._post("mmpaymkttransfers/send_coupon", data=data)

    def query_stock(self, stock_id, op_user_id=None, device_info=None):
        """
        查询代金券批次

        :param stock_id: 代金券批次 ID
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :return: 返回的结果信息
        """
        data = {
            "appid": self.appid,
            "coupon_stock_id": stock_id,
            "op_user_id": op_user_id,
            "device_info": device_info,
            "version": "1.0",
            "type": "XML",
        }
        return self._post("mmpaymkttransfers/query_coupon_stock", data=data)

    def query_coupon(self, coupon_id, user_id, op_user_id=None, device_info=None):
        """
        查询代金券信息

        :param coupon_id: 代金券 ID
        :param user_id: 用户在公众号下的 openid
        :param op_user_id: 可选，操作员账号，默认为商户号
        :param device_info: 可选，微信支付分配的终端设备号
        :return: 返回的结果信息
        """
        data = {
            "coupon_id": coupon_id,
            "openid": user_id,
            "appid": self.appid,
            "op_user_id": op_user_id,
            "device_info": device_info,
            "version": "1.0",
            "type": "XML",
        }
        return self._post("promotion/query_coupon", data=data)



# -*- coding: utf-8 -*-


from wechatpy.client.api.card import WeChatCard  # NOQA
from wechatpy.client.api.customservice import WeChatCustomService  # NOQA
from wechatpy.client.api.datacube import WeChatDataCube  # NOQA
from wechatpy.client.api.device import WeChatDevice  # NOQA
from wechatpy.client.api.group import WeChatGroup  # NOQA
from wechatpy.client.api.invoice import WeChatInvoice  # NOQA
from wechatpy.client.api.jsapi import WeChatJSAPI  # NOQA
from wechatpy.client.api.material import WeChatMaterial  # NOQA
from wechatpy.client.api.media import WeChatMedia  # NOQA
from wechatpy.client.api.menu import WeChatMenu  # NOQA
from wechatpy.client.api.merchant import WeChatMerchant  # NOQA
from wechatpy.client.api.message import WeChatMessage  # NOQA
from wechatpy.client.api.misc import WeChatMisc  # NOQA
from wechatpy.client.api.poi import WeChatPoi  # NOQA
from wechatpy.client.api.qrcode import WeChatQRCode  # NOQA
from wechatpy.client.api.scan import WeChatScan  # NOQA
from wechatpy.client.api.semantic import WeChatSemantic  # NOQA
from wechatpy.client.api.shakearound import WeChatShakeAround  # NOQA
from wechatpy.client.api.tag import WeChatTag  # NOQA
from wechatpy.client.api.template import WeChatTemplate  # NOQA
from wechatpy.client.api.user import WeChatUser  # NOQA
from wechatpy.client.api.wifi import WeChatWiFi  # NOQA
from wechatpy.client.api.wxa import WeChatWxa  # NOQA
from wechatpy.client.api.marketing import WeChatMarketing  # NOQA
from wechatpy.client.api.cloud import WeChatCloud  # NOQA



# -*- coding: utf-8 -*-
import json
import time
import logging
import requests

from wechatpy.client.base import BaseWeChatClient
from wechatpy.exceptions import WeChatClientException
from wechatpy.work.services import api

logger = logging.getLogger(__name__)


class WeChatServiceClient(BaseWeChatClient):
    """
    注意：access_token在第三方应用变更为suite_access_token参数
    """

    API_BASE_URL = "https://qyapi.weixin.qq.com/cgi-bin/"

    auth = api.WeChatAuth()
    miniprogram = api.WeChatMiniProgram()

    def __init__(
        self,
        corp_id,
        suite_id,
        suite_secret,
        suite_ticket,
        access_token=None,
        session=None,
        timeout=None,
        auto_retry=True,
    ):
        self.corp_id = corp_id
        self.suite_id = suite_id
        self.suite_secret = suite_secret
        self.suite_ticket = suite_ticket
        super().__init__(corp_id, access_token, session, timeout, auto_retry)

    @property
    def access_token_key(self):
        return f"services_{self.corp_id}_{self.suite_id}_access_token"

    def _fetch_access_token(self, url, params):
        """The real fetch access token"""
        logger.info("Fetching access token")
        res = self._http.post(url=url, json=params)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )
        result = res.json()
        if "errcode" in result and result["errcode"] != 0:
            raise WeChatClientException(
                result["errcode"],
                result["errmsg"],
                client=self,
                request=res.request,
                response=res,
            )

        expires_in = 7200
        if "expires_in" in result:
            expires_in = result["expires_in"]
        self.session.set(self.access_token_key, result["suite_access_token"], expires_in)
        self.expires_at = int(time.time()) + expires_in
        return result

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            api_base_url = kwargs.pop("api_base_url", self.API_BASE_URL)
            url = f"{api_base_url}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if "params" not in kwargs:
            kwargs["params"] = {}
        if isinstance(kwargs["params"], dict) and "suite_access_token" not in kwargs["params"]:
            kwargs["params"]["suite_access_token"] = self.access_token
        if isinstance(kwargs.get("data", ""), dict):
            body = json.dumps(kwargs["data"], ensure_ascii=False)
            body = body.encode("utf-8")
            kwargs["data"] = body

        kwargs["timeout"] = kwargs.get("timeout", self.timeout)
        result_processor = kwargs.pop("result_processor", None)
        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )

        return self._handle_result(res, method, url, result_processor, **kwargs)

    def fetch_access_token(self):
        """Fetch access token"""
        return self._fetch_access_token(
            url="https://qyapi.weixin.qq.com/cgi-bin/service/get_suite_token",
            params={"suite_id": self.suite_id, "suite_secret": self.suite_secret, "suite_ticket": self.suite_ticket},
        )



# -*- coding: utf-8 -*-
"""
    wechatpy.exceptions
    ~~~~~~~~~~~~~~~~~~~~

    Basic exceptions definition.

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""


class WeChatException(Exception):
    """Base exception for wechatpy"""

    def __init__(self, errcode, errmsg):
        """
        :param errcode: Error code
        :param errmsg: Error message
        """
        self.errcode = errcode
        self.errmsg = errmsg

    def __str__(self):
        s = f"Error code: {self.errcode}, message: {self.errmsg}"
        return s

    def __repr__(self):
        _repr = f"{self.__class__.__name__}({self.errcode}, {self.errmsg})"
        return _repr


class WeChatClientException(WeChatException):
    """WeChat API client exception class"""

    def __init__(self, errcode, errmsg, client=None, request=None, response=None):
        super().__init__(errcode, errmsg)
        self.client = client
        self.request = request
        self.response = response


class InvalidSignatureException(WeChatException):
    """Invalid signature exception class"""

    def __init__(self, errcode=-40001, errmsg="Invalid signature"):
        super().__init__(errcode, errmsg)


class APILimitedException(WeChatClientException):
    """WeChat API call limited exception class"""

    pass


class InvalidAppIdException(WeChatException):
    """Invalid app_id exception class"""

    def __init__(self, errcode=-40005, errmsg="Invalid AppId"):
        super().__init__(errcode, errmsg)


class InvalidMchIdException(WeChatException):
    """Invalid mch_id exception class"""

    def __init__(self, errcode=-40006, errmsg="Invalid MchId"):
        super().__init__(errcode, errmsg)


class WeChatOAuthException(WeChatClientException):
    """WeChat OAuth API exception class"""

    pass


class WeChatComponentOAuthException(WeChatClientException):
    """WeChat Component OAuth API exception class"""

    pass


class WeChatPayException(WeChatClientException):
    """WeChat Pay API exception class"""

    def __init__(
        self,
        return_code,
        result_code=None,
        return_msg=None,
        errcode=None,
        errmsg=None,
        client=None,
        request=None,
        response=None,
    ):
        """
        :param return_code: 返回状态码
        :param result_code: 业务结果
        :param return_msg: 返回信息
        :param errcode: 错误代码
        :param errmsg: 错误代码描述
        """
        super().__init__(errcode, errmsg, client, request, response)
        self.return_code = return_code
        self.result_code = result_code
        self.return_msg = return_msg

    def __str__(self):
        _str = f"Error code: {self.return_code}, message: {self.return_msg}. Pay Error code: {self.errcode}, message: {self.errmsg}"
        return _str

    def __repr__(self):
        _repr = f"{self.__class__.__name__}({self.return_code}, {self.return_msg}). Pay({self.errcode}, {self.errmsg})"
        return _repr



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatService(BaseWeChatAPI):
    """
    应用授权（服务商、第三方应用开发相关）

    https://work.weixin.qq.com/api/doc#90001/90143/90597

    新的授权体系有部分接口未实现，欢迎提交 PR。
    """

    def get_provider_token(self, provider_secret):
        """
        获取服务商凭证

        https://work.weixin.qq.com/api/doc#90001/90143/91200

        :param provider_secret: 服务商的secret，在服务商管理后台可见
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "service/get_provider_token",
            data={
                "corpid": self._client.corp_id,
                "provider_secret": provider_secret,
            },
        )

    def get_suite_token(self, suite_id, suite_secret, suite_ticket):
        """
        获取第三方应用凭证

        https://work.weixin.qq.com/api/doc#90001/90143/9060

        :param suite_id: 以ww或wx开头应用id（对应于旧的以tj开头的套件id）
        :param suite_secret:  应用secret
        :param suite_ticket: 企业微信后台推送的ticket
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "service/get_suite_token",
            data={
                "suite_id": suite_id,
                "suite_secret": suite_secret,
                "suite_ticket": suite_ticket,
            },
        )

    def get_login_info(self, auth_code, provider_access_token=None):
        """
        获取企业号登录用户信息

        详情请参考
        https://work.weixin.qq.com/api/doc/90001/90143/91125

        :param provider_access_token: 服务提供商的 accesstoken
        :param auth_code: OAuth 2.0 授权企业号管理员登录产生的 code
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "service/get_login_info",
            params={
                "provider_access_token": provider_access_token,
            },
            data={
                "auth_code": auth_code,
            },
        )



# -*- coding: utf-8 -*-


from wechatpy.work.client.api.agent import WeChatAgent  # NOQA
from wechatpy.work.client.api.appchat import WeChatAppChat  # NOQA
from wechatpy.work.client.api.batch import WeChatBatch  # NOQA
from wechatpy.work.client.api.calendar import WeChatCalendar  # NOQA
from wechatpy.work.client.api.department import WeChatDepartment  # NOQA
from wechatpy.work.client.api.email import WeChatEMail  # NOQA
from wechatpy.work.client.api.external_contact import WeChatExternalContact  # NOQA
from wechatpy.work.client.api.external_contact_group_chat import WeChatExternalContactGroupChat
from wechatpy.work.client.api.invoice import WeChatInvoice  # NOQA
from wechatpy.work.client.api.jsapi import WeChatJSAPI  # NOQA
from wechatpy.work.client.api.media import WeChatMedia  # NOQA
from wechatpy.work.client.api.menu import WeChatMenu  # NOQA
from wechatpy.work.client.api.message import WeChatMessage  # NOQA
from wechatpy.work.client.api.misc import WeChatMisc  # NOQA
from wechatpy.work.client.api.oa import WeChatOA  # NOQA
from wechatpy.work.client.api.oauth import WeChatOAuth  # NOQA
from wechatpy.work.client.api.schedule import WeChatSchedule  # NOQA
from wechatpy.work.client.api.service import WeChatService  # NOQA
from wechatpy.work.client.api.tag import WeChatTag  # NOQA
from wechatpy.work.client.api.user import WeChatUser  # NOQA
from wechatpy.work.client.api.kf import WeChatKF  # NOQA
from wechatpy.work.client.api.kf_message import WeChatKFMessage  # NOQA



# -*- coding: utf-8 -*-

import re
from urllib.parse import urlencode

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI
from wechatpy.utils import random_string


class WeChatMessage(BaseWeChatAPI):

    OPENID_RE = re.compile(r"^[\w\-]{28}$", re.I)

    def _send_custom_message(self, data, account=None):
        data = data or {}
        if account:
            data["customservice"] = {"kf_account": account}
        return self._post("message/custom/send", data=data)

    def send_text(self, user_id, content, account=None):
        """
        发送文本消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param content: 消息正文
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.send_text('openid', 'text')

        """
        data = {"touser": user_id, "msgtype": "text", "text": {"content": content}}
        return self._send_custom_message(data, account=account)

    def send_image(self, user_id, media_id, account=None):
        """
        发送图片消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param media_id: 图片的媒体ID。 可以通过 :func:`upload_media` 上传。
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.send_image('openid', 'media_id')

        """
        data = {"touser": user_id, "msgtype": "image", "image": {"media_id": media_id}}
        return self._send_custom_message(data, account=account)

    def send_voice(self, user_id, media_id, account=None):
        """
        发送语音消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param media_id: 发送的语音的媒体ID。 可以通过 :func:`upload_media` 上传。
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.send_voice('openid', 'media_id')

        """
        data = {"touser": user_id, "msgtype": "voice", "voice": {"media_id": media_id}}
        return self._send_custom_message(data, account=account)

    def send_video(self, user_id, media_id, title=None, description=None, account=None):
        """
        发送视频消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param media_id: 发送的视频的媒体ID。 可以通过 :func:`upload_media` 上传。
        :param title: 视频消息的标题
        :param description: 视频消息的描述
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.send_video('openid', 'media_id', 'title', 'description')
        """
        video_data = {
            "media_id": media_id,
        }
        if title:
            video_data["title"] = title
        if description:
            video_data["description"] = description

        data = {"touser": user_id, "msgtype": "video", "video": video_data}
        return self._send_custom_message(data, account=account)

    def send_music(
        self,
        user_id,
        url,
        hq_url,
        thumb_media_id,
        title=None,
        description=None,
        account=None,
    ):
        """
        发送音乐消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param url: 音乐链接
        :param hq_url: 高品质音乐链接，wifi环境优先使用该链接播放音乐
        :param thumb_media_id: 缩略图的媒体ID。 可以通过 :func:`upload_media` 上传。
        :param title: 音乐标题
        :param description: 音乐描述
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包
        """
        music_data = {
            "musicurl": url,
            "hqmusicurl": hq_url,
            "thumb_media_id": thumb_media_id,
        }
        if title:
            music_data["title"] = title
        if description:
            music_data["description"] = description

        data = {"touser": user_id, "msgtype": "music", "music": music_data}
        return self._send_custom_message(data, account=account)

    def send_articles(self, user_id, articles, account=None):
        """
        发送图文消息

        详情请参考
        http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param articles: 一个包含至多10个图文的数组, 或者微信图文消息素材 media_id
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包
        """
        if isinstance(articles, (tuple, list)):
            articles_data = []
            for article in articles:
                articles_data.append(
                    {
                        "title": article["title"],
                        "description": article["description"],
                        "url": article["url"],
                        "picurl": article.get("image", article.get("picurl")),
                    }
                )
            data = {
                "touser": user_id,
                "msgtype": "news",
                "news": {"articles": articles_data},
            }
        else:
            data = {
                "touser": user_id,
                "msgtype": "mpnews",
                "mpnews": {
                    "media_id": articles,
                },
            }
        return self._send_custom_message(data, account=account)

    def send_link(self, user_id, article, account=None):
        """
        发送图文消息(单图文)

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/customer-message/customerServiceMessage.send.html#method-http

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param article:
        :return: 返回的 JSON 数据包
        """

        data = {
            "touser": user_id,
            "msgtype": "link",
            "link": {
                "title": article["title"],
                "description": article["description"],
                "url": article["url"],
                "thumb_url": article.get("image", article.get("picurl", article.get("thumb_url"))),
            },
        }
        return self._send_custom_message(data, account=account)

    def send_card(self, user_id, card_id, card_ext=None, account=None):
        """
        发送卡券消息

        详情请参参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140547

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param card_id: 卡券 ID
        :param card_ext: 可选，卡券扩展信息
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包
        """
        wxcard = {
            "card_id": card_id,
        }
        if card_ext:
            wxcard["card_ext"] = card_ext
        data = {
            "touser": user_id,
            "msgtype": "wxcard",
            "wxcard": wxcard,
        }
        return self._send_custom_message(data, account=account)

    def send_mini_program_page(self, user_id, miniprogrampage, account=None):
        """发送小程序卡片（要求小程序与公众号已关联）

        详情请参参考
        https://mp.weixin.qq.com/wiki?t=resource/res_main&id=mp1421140547

        :param user_id: 用户 ID openid
        :param miniprogrampage: 小程序卡片信息
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包
        """
        data = {
            "touser": user_id,
            "msgtype": "miniprogrampage",
            "miniprogrampage": miniprogrampage,
        }
        return self._send_custom_message(data, account=account)

    def delete_mass(self, msg_id):
        """
        删除群发消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param msg_id: 要删除的群发消息 ID
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.delete_mass('message id')

        """
        return self._post("message/mass/delete", data={"msg_id": msg_id})

    def _send_mass_message(
        self,
        tag_or_users,
        msg_type,
        msg,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        data = {
            "msgtype": msg_type,
            "send_ignore_reprint": send_ignore_reprint,
        }
        if client_msg_id is not None:
            data["clientmsgid"] = client_msg_id
        if not preview:
            if isinstance(tag_or_users, (tuple, list)):
                # send by user ids
                data["touser"] = tag_or_users
                endpoint = "message/mass/send"
            else:
                # send by tag_id
                data["filter"] = {
                    "tag_id": tag_or_users,
                    "is_to_all": is_to_all,
                }
                endpoint = "message/mass/sendall"
        else:
            if not isinstance(tag_or_users, str):
                raise ValueError("tag_or_users should be str")
            # 预览接口
            if self.OPENID_RE.match(tag_or_users):
                # 按照 openid 预览群发
                data["touser"] = tag_or_users
            else:
                # 按照微信号预览群发
                data["towxname"] = tag_or_users
            endpoint = "message/mass/preview"

        data.update(msg)
        return self._post(endpoint, data=data)

    def send_mass_text(
        self,
        content,
        tag_or_users=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发文本消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param content: 消息正文
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        return self._send_mass_message(
            tag_or_users,
            "text",
            {"text": {"content": content}},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def send_mass_image(
        self,
        media_id,
        tag_or_users=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发图片消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param media_id: 图片的媒体 ID。 可以通过 :func:`upload_media` 上传。
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        return self._send_mass_message(
            tag_or_users,
            "image",
            {"image": {"media_id": media_id}},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def send_mass_voice(
        self,
        media_id,
        tag_or_users=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发语音消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param media_id: 语音的媒体 ID。可以通过 :func:`upload_media` 上传。
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        return self._send_mass_message(
            tag_or_users,
            "voice",
            {"voice": {"media_id": media_id}},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def send_mass_video(
        self,
        media_id,
        tag_or_users=None,
        title=None,
        description=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发视频消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param media_id: 视频的媒体 ID。可以通过 :func:`upload_video` 上传。
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param title: 视频标题
        :param description: 视频描述
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        video_data = {"media_id": media_id}
        if title:
            video_data["title"] = title
        if description:
            video_data["description"] = description
        return self._send_mass_message(
            tag_or_users,
            "mpvideo",
            {"mpvideo": video_data},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def send_mass_article(
        self,
        media_id,
        tag_or_users=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发图文消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param media_id: 图文的媒体 ID。可以通过 :func:`upload_articles` 上传。
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        return self._send_mass_message(
            tag_or_users,
            "mpnews",
            {"mpnews": {"media_id": media_id}},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def get_mass(self, msg_id):
        """
        查询群发消息发送状态

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param msg_id: 群发消息后返回的消息id
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.message.get_mass('mass message id')

        """
        return self._post("message/mass/get", data={"msg_id": msg_id})

    def send_template(self, user_id, template_id, data, url=None, mini_program=None):
        """
        发送模板消息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#5

        :param user_id: 用户 ID 。 就是你收到的 `Message` 的 source
        :param template_id: 模板 ID。在公众平台线上模板库中选用模板获得
        :param url: 链接地址
        :param data: 模板消息数据
        :param mini_program: 跳小程序所需数据, 如：`{'appid': 'appid', 'pagepath': 'index?foo=bar'}`
        :return: 返回的 JSON 数据包
        """
        tpl_data = optionaldict(
            touser=user_id,
            template_id=template_id,
            url=url,
            miniprogram=mini_program,
            data=data,
        )
        return self._post("message/template/send", data=tpl_data)

    def get_autoreply_info(self):
        """
        获取自动回复规则

        详情请参考
        http://mp.weixin.qq.com/wiki/7/7b5789bb1262fb866d01b4b40b0efecb.html

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            info = client.message.get_autoreply_info()

        """
        return self._get("get_current_autoreply_info")

    def send_mass_card(
        self,
        card_id,
        tag_or_users=None,
        is_to_all=False,
        preview=False,
        send_ignore_reprint=0,
        client_msg_id=None,
    ):
        """
        群发卡券消息

        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1481187827_i0l21

        :param card_id: 卡券 ID
        :param tag_or_users: 值为整型数字时为按标签群发，值为列表/元组时为按 OpenID 列表群发
                               当 is_to_all 为 True 时，默认不传或传入 None 即对所有用户发送。
        :param is_to_all: 用于设定是否向全部用户发送，值为true或false，选择true该消息群发给所有用户
                          选择false可根据group_id发送给指定群组的用户
        :type is_to_all: bool
        :param preview: 是否发送预览，此时 tag_or_users 参数应为一个openid字符串
        :type preview: bool
        :param send_ignore_reprint: 指定待群发的文章被判定为转载时，是否继续群发。
                                    当 send_ignore_reprint 参数设置为1时，文章被判定为转载时，且原创文允许转载时，将继续进行群发操作。
                                    当 send_ignore_reprint 参数设置为0时，文章被判定为转载时，将停止群发操作。
                                    send_ignore_reprint 默认为0。
        :type send_ignore_reprint: int
        :param client_msg_id: 开发者侧群发 msgid，长度限制 64 字节
        :type client_msg_id: str

        :return: 返回的 JSON 数据包
        """
        return self._send_mass_message(
            tag_or_users,
            "wxcard",
            {"wxcard": {"card_id": card_id}},
            is_to_all,
            preview,
            send_ignore_reprint,
            client_msg_id,
        )

    def get_subscribe_authorize_url(self, scene, template_id, redirect_url, reserved=None):
        """
        构造请求用户授权的url
        详情请参阅：
        https://mp.weixin.qq.com/wiki?id=mp1500374289_66bvB

        :param scene: 订阅场景值，开发者可以填0-10000的整形值，用来标识订阅场景值
        :type scene: int
        :param template_id: 订阅消息模板ID，登录公众平台后台，在接口权限列表处可查看订阅模板ID
        :param redirect_url: 授权后重定向的回调地址
        :param reserved: 用于保持请求和回调的状态，授权请后原样带回给第三方。该参数可用于防止csrf攻击。若不指定则随机生成。
        """
        if reserved is None:
            reserved = random_string()
        base_url = "https://mp.weixin.qq.com/mp/subscribemsg"
        params = [
            ("action", "get_confirm"),
            ("appid", self.appid),
            ("scene", scene),
            ("template_id", template_id),
            ("redirect_url", redirect_url),
            ("reserved", reserved),
        ]
        encoded_params = urlencode(params)
        return f"{base_url}?{encoded_params}#wechat_redirect"

    def send_subscribe_template(self, openid, template_id, scene, title, data, url=None):
        """
        一次性订阅消息，通过API推送订阅模板消息给到授权微信用户。
        详情请参阅：
        https://mp.weixin.qq.com/wiki?id=mp1500374289_66bvB

        :param openid: 填接收消息的用户openid
        :param template_id: 订阅消息模板ID
        :param scene: 订阅场景值，开发者可以填0-10000的整形值，用来标识订阅场景值
        :type scene: int
        :param title: 消息标题，15字以内
        :param data: 消息正文，value为消息内容，color为颜色，200字以内
        :type data: dict
        :param url: 点击消息跳转的链接，需要有ICP备案
        """
        post_data = {
            "touser": openid,
            "template_id": template_id,
            "url": url,
            "scene": scene,
            "title": title,
            "data": data,
        }
        if url is not None:
            post_data["url"] = url
        return self._post(
            "message/template/subscribe",
            data=post_data,
        )

    def send_subscribe_message(self, openid, template_id, data, miniprogram=None, page=None):
        """
        发送服务号订阅消息
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#send%E5%8F%91%E9%80%81%E8%AE%A2%E9%98%85%E9%80%9A%E7%9F%A5

        备注：page 和 miniprogram 同时不填，无跳转；page 和 miniprogram 同时填写，优先跳转小程序
        """
        post_data = optionaldict(
            touser=openid,
            template_id=template_id,
            data=data,
            page=page,
            miniprogram=miniprogram,
        )
        return self._post("message/subscribe/bizsend", data=post_data)

    def send_msg_menu(self, openid, msgmenu, account=None):
        """
        发送菜单消息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#7

        :param openid: 填接收消息的用户openid
        :param msgmenu: 菜单字典
        :param account: 可选，客服账号
        :return: 返回的 JSON 数据包
        """
        data = {"touser": openid, "msgtype": "msgmenu", "msgmenu": msgmenu}
        return self._send_custom_message(data, account=account)



# -*- coding: utf-8 -*-
"""
    wechatpy.crypto
    ~~~~~~~~~~~~~~~~

    This module provides some crypto tools for WeChat and WeChat work

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""

import json
import time
import base64
import hashlib

from wechatpy.utils import to_text, to_binary, WeChatSigner
from wechatpy.exceptions import (
    InvalidAppIdException,
    InvalidMchIdException,
    InvalidSignatureException,
)
from wechatpy.crypto.base import BasePrpCrypto, WeChatCipher, BaseRefundCrypto
from wechatpy.crypto.pkcs7 import PKCS7Encoder


def _get_signature(token, timestamp, nonce, encrypt):
    signer = WeChatSigner()
    signer.add_data(token, timestamp, nonce, encrypt)
    return signer.signature


class PrpCrypto(BasePrpCrypto):
    def encrypt(self, text, app_id):
        return self._encrypt(text, app_id)

    def decrypt(self, text, app_id):
        return self._decrypt(text, app_id, InvalidAppIdException)


class BaseWeChatCrypto:
    def __init__(self, token, encoding_aes_key, _id):
        encoding_aes_key = to_binary(encoding_aes_key + "=")
        self.key = base64.b64decode(encoding_aes_key)
        assert len(self.key) == 32
        self.token = token
        self._id = _id

    def _check_signature(self, signature, timestamp, nonce, echo_str, crypto_class=None):
        _signature = _get_signature(self.token, timestamp, nonce, echo_str)
        if _signature != signature:
            raise InvalidSignatureException()
        pc = crypto_class(self.key)
        return pc.decrypt(echo_str, self._id)

    def _encrypt_message(self, msg, nonce, timestamp=None, crypto_class=None):
        from wechatpy.replies import BaseReply

        xml = """<xml>
<Encrypt><![CDATA[{encrypt}]]></Encrypt>
<MsgSignature><![CDATA[{signature}]]></MsgSignature>
<TimeStamp>{timestamp}</TimeStamp>
<Nonce><![CDATA[{nonce}]]></Nonce>
</xml>"""
        if isinstance(msg, BaseReply):
            msg = msg.render()
        timestamp = timestamp or to_text(int(time.time()))
        pc = crypto_class(self.key)
        encrypt = to_text(pc.encrypt(msg, self._id))
        signature = _get_signature(self.token, timestamp, nonce, encrypt)
        return to_text(xml.format(encrypt=encrypt, signature=signature, timestamp=timestamp, nonce=nonce))

    def _decrypt_message(self, msg, signature, timestamp, nonce, crypto_class=None):
        if not isinstance(msg, dict):
            import xmltodict

            msg = xmltodict.parse(to_text(msg))["xml"]

        encrypt = msg["Encrypt"]
        _signature = _get_signature(self.token, timestamp, nonce, encrypt)
        if _signature != signature:
            raise InvalidSignatureException()
        pc = crypto_class(self.key)
        return pc.decrypt(encrypt, self._id)


class WeChatCrypto(BaseWeChatCrypto):
    def __init__(self, token, encoding_aes_key, app_id):
        super().__init__(token, encoding_aes_key, app_id)
        self.app_id = app_id

    def encrypt_message(self, msg, nonce, timestamp=None):
        return self._encrypt_message(msg, nonce, timestamp, PrpCrypto)

    def decrypt_message(self, msg, signature, timestamp, nonce):
        return self._decrypt_message(msg, signature, timestamp, nonce, PrpCrypto)


class WeChatWxaCrypto:
    def __init__(self, key, iv, app_id):
        self.cipher = WeChatCipher(base64.b64decode(key), base64.b64decode(iv))
        self.app_id = app_id

    def decrypt_message(self, msg):
        raw_data = base64.b64decode(msg)
        decrypted = self.cipher.decrypt(raw_data)
        plaintext = PKCS7Encoder.decode(decrypted)
        decrypted_msg = json.loads(to_text(plaintext))
        if decrypted_msg["watermark"]["appid"] != self.app_id:
            raise InvalidAppIdException()
        return decrypted_msg


class RefundCrypto(BaseRefundCrypto):
    def encrypt(self, text):
        return self._encrypt(text)

    def decrypt(self, text):
        return self._decrypt(text)


class WeChatRefundCrypto:
    def __init__(self, key):
        self.key = to_binary(hashlib.md5(to_binary(key)).hexdigest())
        assert len(self.key) == 32

    def _decrypt_message(self, msg, appid, mch_id, crypto_class=None):
        import xmltodict

        if not isinstance(msg, dict):
            msg = xmltodict.parse(to_text(msg))["xml"]

        req_info = msg["req_info"]
        if msg["appid"] != appid:
            raise InvalidAppIdException()
        if msg["mch_id"] != mch_id:
            raise InvalidMchIdException()
        pc = crypto_class(self.key)
        ret = pc.decrypt(req_info)
        return xmltodict.parse(to_text(ret))["root"]

    def decrypt_message(self, msg, appid, mch_id):
        return self._decrypt_message(msg, appid, mch_id, RefundCrypto)



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantStock(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def add(self, product_id, quantity, sku_info=""):
        return self._post(
            "merchant/stock/add",
            data={"product_id": product_id, "quantity": quantity, "sku_info": sku_info},
        )

    def reduce(self, product_id, quantity, sku_info=""):
        return self._post(
            "merchant/stock/reduce",
            data={"product_id": product_id, "quantity": quantity, "sku_info": sku_info},
        )



# -*- coding: utf-8 -*-


class BaseWeChatAPI:
    """WeChat API base class"""

    def __init__(self, client=None):
        self._client = client

    def _get(self, url, **kwargs):
        if getattr(self, "API_BASE_URL", None):
            kwargs["api_base_url"] = self.API_BASE_URL
        return self._client.get(url, **kwargs)

    def _post(self, url, **kwargs):
        if getattr(self, "API_BASE_URL", None):
            kwargs["api_base_url"] = self.API_BASE_URL
        return self._client.post(url, **kwargs)

    @property
    def access_token(self):
        return self._client.access_token

    @property
    def session(self):
        return self._client.session

    @property
    def appid(self):
        return self._client.appid

    @property
    def secret(self):
        return self._client.secret



# -*- coding: utf-8 -*-
from operator import itemgetter


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatCard(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def create(self, card_data):
        """
        创建卡券

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Create_a_Coupon_Voucher_or_Card.html#8

        :param card_data: 卡券信息
        :return: 创建的卡券 ID
        """
        result = self._post("card/create", data=card_data, result_processor=itemgetter("card_id"))
        return result

    def batch_add_locations(self, location_data):
        """
        ⚠️已废弃
        批量导入门店信息

        :param location_data: 门店信息
        :return: 门店 ID 列表，插入失败的门店元素值为 -1
        """
        result = self._post(
            "card/location/batchadd",
            data=location_data,
            result_processor=itemgetter("location_id_list"),
        )
        return result

    def batch_get_locations(self, offset=0, count=0):
        """
        ⚠️已废弃
        批量获取门店信息
        """
        return self._post("card/location/batchget", data={"offset": offset, "count": count})

    def get_colors(self):
        """
        获得卡券的最新颜色列表，用于创建卡券
        目前微信提供包括十四种色值供开发者使用。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Create_a_Coupon_Voucher_or_Card.html#7

        :return: 颜色列表
        """
        return [
            "Color010",  # "#63b359"
            "Color020",  # "#2c9f67"
            "Color030",  # "#509fc9"
            "Color040",  # "#5885cf"
            "Color050",  # "#9062c0"
            "Color060",  # "#d09a45"
            "Color070",  # "#e4b138"
            "Color080",  # "#ee903c"
            "Color081",  # "#f08500"
            "Color082",  # "#a9d92d"
            "Color090",  # "#dd6549"
            "Color100",  # "#cc463d"
            "Color101",  # "#cf3e36"
            "Color102",  # "#5E6671"
        ]

    def create_qrcode(self, qrcode_data):
        """
        创建卡券二维码

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Distributing_Coupons_Vouchers_and_Cards.html#0

        :param qrcode_data: 二维码信息
        :return: 二维码 ticket，可使用 :func:show_qrcode 换取二维码文件
        """
        result = self._post(
            "card/qrcode/create",
            data=qrcode_data,
            result_processor=itemgetter("ticket"),
        )
        return result

    def create_landingpage(self, buffer_data):
        """
        创建货架

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Distributing_Coupons_Vouchers_and_Cards.html#3

        """
        result = self._post("card/landingpage/create", data=buffer_data)
        return result

    def get_html(self, card_id):
        """
        图文消息群发卡券

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Distributing_Coupons_Vouchers_and_Cards.html#6
        """
        result = self._post(
            "card/mpnews/gethtml",
            data={"card_id": card_id},
            result_processor=itemgetter("content"),
        )
        return result

    def consume_code(self, code, card_id=None):
        """
        消耗 code

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Redeeming_a_coupon_voucher_or_card.html#0
        """
        card_data = {"code": code}
        if card_id:
            card_data["card_id"] = card_id
        return self._post("card/code/consume", data=card_data)

    def decrypt_code(self, encrypt_code):
        """
        解码加密的 code

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Redeeming_a_coupon_voucher_or_card.html#0
        """
        result = self._post(
            "card/code/decrypt",
            data={"encrypt_code": encrypt_code},
            result_processor=itemgetter("code"),
        )
        return result

    def delete(self, card_id):
        """
        删除卡券

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#7
        """
        return self._post("card/delete", data={"card_id": card_id})

    def get_code(self, code, card_id=None, check_consume=True):
        """
        查询 code 信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#0
        """
        card_data = {"code": code}
        if card_id:
            card_data["card_id"] = card_id
        if not check_consume:
            card_data["check_consume"] = check_consume
        return self._post("card/code/get", data=card_data)

    def get_card_list(self, openid, card_id=None):
        """
        用于获取用户卡包里的，属于该appid下的卡券。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#1
        """
        card_data = {"openid": openid}
        if card_id:
            card_data["card_id"] = card_id
        return self._post("card/user/getcardlist", data=card_data)

    def batch_get(self, offset=0, count=50, status_list=None):
        """
        批量查询卡券信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#3

        :param offset: 查询卡列表的起始偏移量，从0开始，即offset: 5是指从从列表里的第六个开始读取。
        :param count: 需要查询的卡片的数量（数量最大50）
        :param status_list: 支持开发者拉出指定状态的卡券列表
        """
        if count > 50:
            raise ValueError("count cannot be greater than 50")
        card_data = {"offset": offset, "count": count}
        if status_list:
            card_data["status_list"] = status_list
        return self._post("card/batchget", data=card_data)

    def get(self, card_id):
        """
        查询卡券详情

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#2
        """
        result = self._post("card/get", data={"card_id": card_id}, result_processor=itemgetter("card"))
        return result

    def update_code(self, card_id, old_code, new_code):
        """
        更新卡券 code

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#6

        :param card_id: 卡券ID。自定义 Code 码卡券为必填
        :param old_code: 需变更的 Code 码
        :param new_code: 变更后的有效 Code 码
        """
        return self._post(
            "card/code/update",
            data={"card_id": card_id, "code": old_code, "new_code": new_code},
        )

    def invalid_code(self, code, card_id=None):
        """
        设置卡券失效

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#8
        """
        card_data = {"code": code}
        if card_id:
            card_data["card_id"] = card_id
        return self._post("card/code/unavailable", data=card_data)

    def update(self, card_data):
        """
        更新卡券信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Managing_Coupons_Vouchers_and_Cards.html#4
        """
        return self._post("card/update", data=card_data)

    def set_paycell(self, card_id, is_open):
        """
        设置买单接口

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Create_a_Coupon_Voucher_or_Card.html#12
        """
        return self._post("card/paycell/set", data={"card_id": card_id, "is_open": is_open})

    def set_test_whitelist(self, openids=None, usernames=None):
        """
        设置卡券测试用户白名单

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Distributing_Coupons_Vouchers_and_Cards.html#12
        """
        openids = openids or []
        usernames = usernames or []
        return self._post("card/testwhitelist/set", data={"openid": openids, "username": usernames})

    def activate_membercard(self, membership_number, code, **kwargs):
        """
        激活会员卡 - 接口激活方式

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Create_a_membership_card.html#14

        参数示例：
        {
            "init_bonus": 100,
            "init_bonus_record":"旧积分同步",
            "init_balance": 200,
            "membership_number": "AAA00000001",
            "code": "12312313",
            "card_id": "xxxx_card_id",
            "background_pic_url": "https://mmbiz.qlogo.cn/mmbiz/0?wx_fmt=jpeg",
            "init_custom_field_value1": "xxxxx",
            "init_custom_field_value2": "xxxxx",
            "init_custom_field_value3": "xxxxx"
        }

        返回示例：
        {"errcode":0,   "errmsg":"ok"}

        :param membership_number: 必填，会员卡编号，由开发者填入，作为序列号显示在用户的卡包里。可与Code码保持等值
        :param code: 必填，领取会员卡用户获得的code
        :return: 参见返回示例
        """
        kwargs["membership_number"] = membership_number
        kwargs["code"] = code
        return self._post("card/membercard/activate", data=kwargs)

    def update_membercard(self, code, card_id, **kwargs):
        """
        更新会员信息

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Create_a_membership_card.html#18

        注意事项：
        1.开发者可以同时传入add_bonus和bonus解决由于同步失败带来的幂等性问题。同时传入add_bonus和bonus时
            add_bonus作为积分变动消息中的变量值，而bonus作为卡面上的总积分额度显示。余额变动同理。
        2.开发者可以传入is_notify_bonus控制特殊的积分对账变动不发送消息，余额变动同理。

        参数示例：
        {
            "code": "179011264953",
            "card_id": "p1Pj9jr90_SQRaVqYI239Ka1erkI",
            "background_pic_url": "https://mmbiz.qlogo.cn/mmbiz/0?wx_fmt=jpeg",
            "record_bonus": "消费30元，获得3积分",
            "bonus": 3000,
            "add_bonus": 30,
            "balance": 3000,
            "add_balance": -30,
            "record_balance": "购买焦糖玛琪朵一杯，扣除金额30元。",
            "custom_field_value1": "xxxxx"，
            "custom_field_value2": "xxxxx"，
            "notify_optional": {
                "is_notify_bonus": true,
                "is_notify_balance": true,
                "is_notify_custom_field1":true
            }
        }

        返回示例：
        {
            "errcode": 0,
            "errmsg": "ok",
            "result_bonus": 100,
            "result_balance": 200,
            "openid": "oFS7Fjl0WsZ9AMZqrI80nbIq8xrA"
        }

        :param code: 必填，卡券Code码
        :param card_id: 必填，卡券ID
        :param kwargs: 其他非必填字段，包含则更新对应字段。详情参见微信文档 “7 更新会员信息” 部分
        :return: 参见返回示例
        """
        kwargs.update(
            {
                "code": code,
                "card_id": card_id,
            }
        )
        return self._post("card/membercard/updateuser", data=kwargs)

    def get_membercard_user_info(self, card_id, code):
        """
        查询会员卡的会员信息

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Create_a_membership_card.html#16

        :param card_id: 查询会员卡的 Card ID
        :param code: 所查询用户领取到的 code 值
        :return: 会员信息，包括激活资料、积分信息以及余额等信息
        """
        return self._post(
            "card/membercard/userinfo/get",
            data={
                "card_id": card_id,
                "code": code,
            },
        )

    def add_pay_giftcard(self, base_info, extra_info, is_membercard):
        """
        新增支付后投放卡券的规则，支持支付后领卡，支付后赠券

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Manage_Member_Card.html#4

        :param base_info: 营销规则结构体
        :type base_info: dict
        :param extra_info: 支付规则结构体
        :type extra_info: dict
        :param is_membercard: 本次规则是否是领卡。（领卡传入 True, 赠券传入 False）
        :type is_membercard: bool
        :return: 规则 ID, 设置成功的列表，以及设置失败的列表
        """
        if is_membercard:
            rule_key = "member_rule"
            rule_type = "RULE_TYPE_PAY_MEMBER_CARD"
        else:
            rule_key = "single_pay"
            rule_type = "RULE_TYPE_SINGLE_PAY"
        return self._post(
            "card/paygiftcard/add",
            data={
                "rule_info": {
                    "type": rule_type,
                    "base_info": base_info,
                    rule_key: extra_info,
                }
            },
        )

    def del_pay_giftcard(self, rule_id):
        """
        删除支付后投放卡券的规则

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Manage_Member_Card.html#4

        :param rule_id: 支付即会员的规则 ID
        """
        return self._post(
            "card/paygiftcard/delete",
            data={
                "rule_id": rule_id,
            },
        )

    def get_pay_giftcard(self, rule_id):
        """
        查询支付后投放卡券的规则

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Manage_Member_Card.html#4

        :param rule_id: 支付即会员的规则 ID
        :return: 支付后投放卡券的规则
        :rtype: dict
        """
        return self._post(
            "card/paygiftcard/getbyid",
            data={
                "rule_id": rule_id,
            },
            result_processor=itemgetter("rule_info"),
        )

    def batch_get_pay_giftcard(self, effective=True, offset=0, count=10):
        """
        批量查询支付后投放卡券的规则

        详情请参见
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Manage_Member_Card.html#4

        :param effective: 是否仅查询生效的规则
        :type effective: bool
        :param offset: 起始偏移量
        :type offset: int
        :param count: 查询的数量
        :type count: int
        :return: 支付后投放卡券规则的总数，以及查询到的列表
        """
        return self._post(
            "card/paygiftcard/batchget",
            data={
                "type": "RULE_TYPE_PAY_MEMBER_CARD",
                "effective": effective,
                "offset": offset,
                "count": count,
            },
        )

    def update_movie_ticket(
        self,
        code,
        ticket_class,
        show_time,
        duration,
        screening_room,
        seat_number,
        card_id=None,
    ):
        """
        更新电影票

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Special_ticket.html#7
        """
        ticket = {
            "code": code,
            "ticket_class": ticket_class,
            "show_time": show_time,
            "duration": duration,
            "screening_room": screening_room,
            "seat_number": seat_number,
        }
        if card_id:
            ticket["card_id"] = card_id
        return self._post("card/movieticket/updateuser", data=ticket)

    def checkin_boardingpass(
        self,
        code,
        passenger_name,
        seat_class,
        etkt_bnr,
        seat="",
        gate="",
        boarding_time=None,
        is_cancel=False,
        qrcode_data=None,
        card_id=None,
    ):
        """
        飞机票接口

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Special_ticket.html#3
        """
        data = {
            "code": code,
            "passenger_name": passenger_name,
            "class": seat_class,
            "etkt_bnr": etkt_bnr,
            "seat": seat,
            "gate": gate,
            "is_cancel": is_cancel,
        }
        if boarding_time:
            data["boarding_time"] = boarding_time
        if qrcode_data:
            data["qrcode_data"] = qrcode_data
        if card_id:
            data["card_id"] = card_id
        return self._post("card/boardingpass/checkin", data=data)

    def update_luckymoney_balance(self, code, balance, card_id=None):
        """
        ⚠️已废弃
        更新红包余额
        """
        card_data = {"code": code, "balance": balance}
        if card_id:
            card_data["card_id"] = card_id
        return self._post("card/luckymoney/updateuserbalance", data=card_data)

    def get_redirect_url(self, url, encrypt_code, card_id):
        """
        获取卡券跳转外链
        """
        from wechatpy.utils import WeChatSigner

        code = self.decrypt_code(encrypt_code)

        signer = WeChatSigner()
        signer.add_data(self.secret)
        signer.add_data(code)
        signer.add_data(card_id)
        signature = signer.signature

        return f"{url}?encrypt_code={encrypt_code}&card_id={card_id}&signature={signature}"

    def deposit_code(self, card_id, codes):
        """
        导入code
        """
        card_data = {"card_id": card_id, "code": codes}
        return self._post("card/code/deposit", data=card_data)

    def get_deposit_count(self, card_id):
        """
        查询导入code数目
        """
        card_data = {
            "card_id": card_id,
        }
        return self._post("card/code/getdepositcount", data=card_data)

    def check_code(self, card_id, codes):
        """
        核查code
        """
        card_data = {"card_id": card_id, "code": codes}
        return self._post("card/code/checkcode", data=card_data)

    def modify_stock(self, card_id, n):
        """
        修改库存
        """
        if n == 0:
            return
        card_data = {
            "card_id": card_id,
        }
        if n > 0:
            card_data["increase_stock_value"] = n
        elif n < 0:
            card_data["reduce_stock_value"] = -n
        return self._post("card/modifystock", data=card_data)

    def get_activate_url(self, card_id, outer_str=None):
        """
        获取开卡插件 Url, 内含调用开卡插件所需的参数
        详情请参考
        https://mp.weixin.qq.com/wiki?id=mp1499332673_Unm7V

        :param card_id: 会员卡的card_id
        :param outer_str: 渠道值，用于统计本次领取的渠道参数
        :return: 内含调用开卡插件所需的参数的 Url
        """
        return self._post(
            "card/membercard/activate/geturl",
            data={
                "card_id": card_id,
                "outer_str": outer_str,
            },
            result_processor=lambda x: x["url"],
        )

    def get_activate_info(self, activate_ticket):
        """
        获取用户开卡时提交的信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Create_a_membership_card.html#16

        :param activate_ticket: 跳转型开卡组件开卡后回调中的激活票据，可以用来获取用户开卡资料
        :return: 用户开卡时填写的字段值
        """
        return self._post(
            "card/membercard/activatetempinfo/get",
            data={
                "activate_ticket": activate_ticket,
            },
            result_processor=itemgetter("info"),
        )

    def set_activate_user_form(self, card_id, **kwargs):
        """
        设置开卡字段接口

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Cards_and_Offer/Membership_Cards/Create_a_membership_card.html#16

        "6 激活会员卡" -> "6.2 一键激活" -> "步骤二：设置开卡字段接口"

        参数示例：
        {
            "card_id": "pbLatjnrwUUdZI641gKdTMJzHGfc",
            "service_statement": {
                "name": "会员守则",
                "url": "https://www.qq.com"
            },
            "bind_old_card": {
                "name": "老会员绑定",
                "url": "https://www.qq.com"
            },
            "required_form": {
                "can_modify"：false,
                "rich_field_list": [
                    {
                        "type": "FORM_FIELD_RADIO",
                        "name": "兴趣",
                        "values": [
                            "钢琴",
                            "舞蹈",
                            "足球"
                        ]
                    },
                    {
                        "type": "FORM_FIELD_SELECT",
                        "name": "喜好",
                        "values": [
                            "郭敬明",
                            "韩寒",
                            "南派三叔"
                        ]
                    },
                    {
                        "type": "FORM_FIELD_CHECK_BOX",
                        "name": "职业",
                        "values": [
                            "赛车手",
                            "旅行家"
                        ]
                    }
                ],
                "common_field_id_list": [
                    "USER_FORM_INFO_FLAG_MOBILE"
                ]
            },
            "optional_form": {
                "can_modify"：false,
                "common_field_id_list": [
                    "USER_FORM_INFO_FLAG_LOCATION",
                    "USER_FORM_INFO_FLAG_BIRTHDAY"
                ],
                "custom_field_list": [
                    "喜欢的电影"
                ]
            }
        }
        common_field_id_list 值见常量 `wechatpy.constants.UserFormInfoFlag`

        :param card_id: 卡券ID
        :param kwargs: 其他非必填参数，见微信文档
        """
        kwargs["card_id"] = card_id
        return self._post("card/membercard/activateuserform/set", data=kwargs)



# -*- coding: utf-8 -*-

import struct
import socket
import base64

from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes

from wechatpy.utils import to_text, to_binary, random_string
from wechatpy.crypto.pkcs7 import PKCS7Encoder


class BaseWeChatCipher:
    def __init__(self, cipher):
        self.cipher = cipher

    def encrypt(self, plaintext):
        encryptor = self.cipher.encryptor()
        return encryptor.update(plaintext) + encryptor.finalize()

    def decrypt(self, ciphertext):
        decryptor = self.cipher.decryptor()
        return decryptor.update(ciphertext) + decryptor.finalize()


class WeChatCipher(BaseWeChatCipher):
    def __init__(self, key, iv=None):
        iv = iv or key[:16]
        super().__init__(Cipher(algorithms.AES(key), modes.CBC(iv)))


class AesEcbCipher(BaseWeChatCipher):
    def __init__(self, key):
        super().__init__(Cipher(algorithms.AES(key), modes.ECB()))


class BasePrpCrypto:
    def __init__(self, key):
        self.cipher = WeChatCipher(key)

    def get_random_string(self):
        return random_string(16)

    def _encrypt(self, text, _id):
        text = to_binary(text)
        tmp_list = []
        tmp_list.append(to_binary(self.get_random_string()))
        length = struct.pack(b"I", socket.htonl(len(text)))
        tmp_list.append(length)
        tmp_list.append(text)
        tmp_list.append(to_binary(_id))

        text = b"".join(tmp_list)
        text = PKCS7Encoder.encode(text)

        ciphertext = to_binary(self.cipher.encrypt(text))
        return base64.b64encode(ciphertext)

    def _decrypt(self, text, _id, exception=None):
        text = to_binary(text)
        plain_text = self.cipher.decrypt(base64.b64decode(text))
        padding = plain_text[-1]
        content = plain_text[16:-padding]
        xml_length = socket.ntohl(struct.unpack(b"I", content[:4])[0])
        xml_content = to_text(content[4 : xml_length + 4])
        from_id = to_text(content[xml_length + 4 :])
        if from_id != _id:
            exception = exception or Exception
            raise exception()
        return xml_content


class BaseRefundCrypto:
    def __init__(self, key):
        self.cipher = AesEcbCipher(key)

    def _encrypt(self, text):
        text = to_binary(text)
        text = PKCS7Encoder.encode(text)

        ciphertext = to_binary(self.cipher.encrypt(text))
        return base64.b64encode(ciphertext)

    def _decrypt(self, text, exception=None):
        text = to_binary(text)
        plain_text = self.cipher.decrypt(base64.b64decode(text))
        padding = plain_text[-1]
        content = plain_text[:-padding]
        return content



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatAppChat(BaseWeChatAPI):
    """发送消息到群聊会话
    https://work.weixin.qq.com/api/doc#90000/90135/90244
    """

    def create(self, chat_id=None, name=None, owner=None, user_list=None):
        """
        创建群聊会话

        详情请参考
        https://work.weixin.qq.com/api/doc#90000/90135/90245

        限制说明：
        只允许企业自建应用调用，且应用的可见范围必须是根部门；
        群成员人数不可超过管理端配置的“群成员人数上限”，且最大不可超过500人；
        每企业创建群数不可超过1000/天；

        :param chat_id: 群聊的唯一标志，不能与已有的群重复；字符串类型，最长32个字符。只允许字符0-9及字母a-zA-Z。如果不填，系统会随机生成群id
        :param name: 群聊名，最多50个utf8字符，超过将截断
        :param owner: 指定群主的id。如果不指定，系统会随机从userlist中选一人作为群主
        :param user_list: 会话成员列表，成员用userid来标识。至少2人，至多500人
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            chatid=chat_id,
            name=name,
            owner=owner,
            userlist=user_list,
        )
        return self._post("appchat/create", data=data)

    def get(self, chat_id):
        """
        获取群聊会话

        详情请参考
        https://work.weixin.qq.com/api/doc#90000/90135/90247

        :param chat_id: 群聊id
        :return: 会话信息
        """
        res = self._get("appchat/get", params={"chatid": chat_id})
        return res["chat_info"]

    def update(self, chat_id, name=None, owner=None, add_user_list=None, del_user_list=None):
        """
        修改群聊会话

        详情请参考
        https://work.weixin.qq.com/api/doc#90000/90135/90246

        :param chat_id: 群聊id
        :param name: 新的群聊名。若不需更新，请忽略此参数。最多50个utf8字符，超过将截断
        :param owner: 新群主的id。若不需更新，请忽略此参数
        :param add_user_list: 会话新增成员列表，成员用userid来标识
        :param del_user_list: 会话退出成员列表，成员用userid来标识
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            chatid=chat_id,
            name=name,
            owner=owner,
            add_user_list=add_user_list,
            del_user_list=del_user_list,
        )
        return self._post("appchat/update", data=data)

    def send(self, chat_id, msg_type, **kwargs):
        """
        应用推送消息

        详情请参考：https://work.weixin.qq.com/api/doc#90000/90135/90248
        :param chat_id: 群聊id
        :param msg_type: 消息类型，可以为text/image/voice/video/file/textcard/news/mpnews/markdown
        :param kwargs: 具体消息类型的扩展参数
        :return:
        """
        data = {"chatid": chat_id, "safe": kwargs.get("safe") or 0}
        data.update(self._build_msg_content(msg_type, **kwargs))

        return self._post("appchat/send", data=data)

    def send_text(self, chat_id, content, safe=0):
        """
        发送文本消息

        详情请参考：https://work.weixin.qq.com/api/doc#90000/90135/90248/文本消息/

        :param chat_id: 群聊id
        :param content: 消息内容
        :param safe: 表示是否是保密消息，0表示否，1表示是，默认0
        :return:
        """
        return self.send(chat_id, "text", safe=safe, content=content)

    def _build_msg_content(self, msgtype="text", **kwargs):
        """
        构造消息内容

        :param content: 消息内容，最长不超过2048个字节
        :param msgtype: 消息类型，可以为text/image/voice/video/file/textcard/news/mpnews/markdown
        :param kwargs: 具体消息类型的扩展参数
        :return:
        """
        data = {"msgtype": msgtype}
        if msgtype == "text":
            data[msgtype] = {"content": kwargs.get("content")}
        elif msgtype == "image" or msgtype == "voice" or msgtype == "file":
            data[msgtype] = {"media_id": kwargs.get("media_id")}
        elif msgtype == "video":
            data[msgtype] = {
                "media_id": kwargs.get("media_id"),
                "title": kwargs.get("title"),
                "description": kwargs.get("description"),
            }
        elif msgtype == "textcard":
            data[msgtype] = {
                "title": kwargs.get("title"),
                "description": kwargs.get("description"),
                "url": kwargs.get("url"),
                "btntxt": kwargs.get("btntxt"),
            }
        elif msgtype == "news":
            # {
            #         "articles" :
            #         [
            #             {
            #                 "title" : "中秋节礼品领取",
            #                 "description" : "今年中秋节公司有豪礼相送",
            #                 "url":"https://zhidao.baidu.com/question/2073647112026042748.html",
            #                 "picurl":"http://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png"
            #              }
            #         ]
            #     }
            data[msgtype] = kwargs
        elif msgtype == "mpnews":
            # {
            #         "articles":[
            #             {
            #                 "title": "地球一小时",
            #                 "thumb_media_id": "biz_get(image)",
            #                 "author": "Author",
            #                 "content_source_url": "https://work.weixin.qq.com",
            #                 "content": "3月24日20:30-21:30 \n办公区将关闭照明一小时，请各部门同事相互转告",
            #                 "digest": "3月24日20:30-21:30 \n办公区将关闭照明一小时"
            #             }
            #          ]
            #     }
            data[msgtype] = kwargs
        elif msgtype == "markdown":
            #  {
            #         "content": "您的会议室已经预定，稍后会同步到`邮箱`
            #                 >**事项详情**
            #                 >事　项：<font color=\"info\">开会</font>
            #                 >组织者：@miglioguan
            #                 >参与者：@miglioguan、@kunliu、@jamdeezhou、@kanexiong、@kisonwang
            #                 >
            #                 >会议室：<font color=\"info\">广州TIT 1楼 301</font>
            #                 >日　期：<font color=\"warning\">2018年5月18日</font>
            #                 >时　间：<font color=\"comment\">上午9:00-11:00</font>
            #                 >
            #                 >请准时参加会议。
            #                 >
            #                 >如需修改会议信息，请点击：[修改会议信息](https://work.weixin.qq.com)"
            #    }
            data[msgtype] = kwargs
        else:
            raise TypeError(f"不能识别的msgtype: {msgtype}")
        return data



# -*- coding: utf-8 -*-


import operator as op

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatCalendar(BaseWeChatAPI):
    """
    https://developer.work.weixin.qq.com/document/path/93624
    """

    def add(self, organizer, summary, color, description="", shares=()):
        """
        创建日历

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93647

        :param organizer: 指定的组织者userid。注意该字段指定后不可更新
        :param summary: 日历标题。1 ~ 128 字符
        :param color: 日历在终端上显示的颜色，RGB颜色编码16进制表示，例如：”#0000FF” 表示纯蓝色
        :param description: 日历描述。0 ~ 512 字符
        :param shares: 日历共享成员列表。最多2000人
        :type shares: list[str]

        :return: 日历ID
        :rtype: str
        """
        data = {
            "calendar": {
                "organizer": organizer,
                "summary": summary,
                "color": color,
                "description": description,
                "shares": [{"userid": userid} for userid in shares],
            }
        }
        return self._post("oa/calendar/add", data=data, result_processor=op.itemgetter("cal_id"))

    def update(self, calendar_id, summary, color, description="", shares=()):
        """
        更新日历

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93647

        :param calendar_id: 日历ID
        :param summary: 日历标题。1 ~ 128 字符
        :param color: 日历在终端上显示的颜色，RGB颜色编码16进制表示，例如：”#0000FF” 表示纯蓝色
        :param description: 日历描述。0 ~ 512 字符
        :param shares: 日历共享成员列表。最多2000人
        :type shares: list[str]
        """
        data = {
            "calendar": {
                "cal_id": calendar_id,
                "summary": summary,
                "color": color,
                "description": description,
                "shares": [{"userid": userid} for userid in shares],
            }
        }
        return self._post("oa/calendar/update", data=data, result_processor=op.itemgetter("cal_id"))

    def get(self, calendar_ids):
        """
        获取日历

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93647

        :param calendar_ids: 日历ID列表。一次最多可获取1000条
        :type calendar_ids: list[str]

        :return: 日历列表
        :rtype: list[dict]
        """
        return self._post(
            "oa/calendar/get",
            data={"cal_id_list": calendar_ids},
            result_processor=op.itemgetter("calendar_list"),
        )

    def delete(self, calendar_id):
        """
        删除日历

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93647

        :param calendar_id: 日历ID
        """
        return self._post("oa/calendar/del", data={"cal_id": calendar_id})



# -*- coding: utf-8 -*-
"""
    wechatpy.parser
    ~~~~~~~~~~~~~~~~
    This module provides functions for parsing WeChat messages

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
import xmltodict

from wechatpy.messages import MESSAGE_TYPES, UnknownMessage
from wechatpy.events import EVENT_TYPES
from wechatpy.utils import to_text


def parse_message(xml):
    """
    解析微信服务器推送的 XML 消息

    :param xml: XML 消息
    :return: 解析成功返回对应的消息或事件，否则返回 ``UnknownMessage``
    """
    if not xml:
        return
    message = xmltodict.parse(to_text(xml))["xml"]
    message_type = message["MsgType"].lower()
    event_type = None
    if message_type == "event" or message_type.startswith("device_"):
        if "Event" in message:
            event_type = message["Event"].lower()
        # special event type for device_event
        if event_type is None and message_type.startswith("device_"):
            event_type = message_type
        elif message_type.startswith("device_"):
            event_type = f"device_{event_type}"

        if event_type == "subscribe" and message.get("EventKey"):
            event_key = message["EventKey"]
            if event_key.startswith(("scanbarcode|", "scanimage|")):
                event_type = "subscribe_scan_product"
                message["Event"] = event_type
            elif event_key.startswith("qrscene_"):
                # Scan to subscribe with scene id event
                event_type = "subscribe_scan"
                message["Event"] = event_type
                message["EventKey"] = event_key[len("qrscene_") :]
        message_class = EVENT_TYPES.get(event_type, UnknownMessage)
    else:
        message_class = MESSAGE_TYPES.get(message_type, UnknownMessage)
    return message_class(message)



# -*- coding: utf-8 -*-
from operator import itemgetter

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatEMail(BaseWeChatAPI):
    """
    企业微信邮箱接口

    https://developer.work.weixin.qq.com/document/path/95486
    """

    def create_group(
        self,
        group_id,
        group_name,
        email_list=None,
        tag_list=None,
        department_list=None,
        group_list=None,
        allow_type=None,
        allow_emaillist=None,
        allow_departmentlist=None,
        allow_taglist=None,
    ):
        """
        该接口用于创建新邮件群组，可以指定群组成员，定义群组使用权限范围

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95510

        :param group_id: 必填，邮件群组 ID，邮箱格式。
        :param group_name: 必填，邮件群组名称，不能与其他群组重名，长度限定64字节。
        :param email_list: 选填，群组内成员邮箱地址，读取成员的 biz_mail 字段，email_list，group_list，department_list，tag_list 至少填写一个，不可同时为空。
        :param tag_list: 选填，群组内包含的标签ID。
        :param department_list: 选填，群组内包含的部门 ID。
        :param group_list: 选填，群组内包含的群组邮箱。
        :param allow_type: 选填，群组使用权限。
            - 0: 企业成员
            - 1：任何人
            - 2：组内成员
            - 3：自定义成员
            当值为 0、1、2时，不得传入 allow_emaillist，allow_departmentlist，allow_taglist。
            当值为 3 时，必须传入 allow_emaillist，allow_departmentlist，allow_taglist至少一项。
        :param allow_emaillist: 选填，允许使用群组群发的成员邮箱地址，读取成员的biz_mail字段。
        :param allow_departmentlist: 选填，允许使用群组群发的部门 ID。
        :param allow_taglist: 选填，允许使用群组群发的标签 ID。
        """
        if not any((email_list, group_list, department_list, tag_list)):
            raise ValueError("email_list, group_list, department_list and tag_list must be a choice.")

        data = {
            "groupid": group_id,
            "groupname": group_name,
            "email_list": {
                "list": email_list or [],
            },
            "tag_list": {
                "list": tag_list or [],
            },
            "department_list": {
                "list": department_list or [],
            },
            "group_list": {
                "list": group_list or [],
            },
        }

        if allow_type is not None:
            if allow_type in {0, 1, 2}:
                data["allow_type"] = allow_type
            elif allow_type == 3:
                if not any((allow_emaillist, allow_departmentlist, allow_taglist)):
                    raise ValueError("allow_emaillist, allow_departmentlist and allow_taglist must be a choice.")
                data.update(
                    {
                        "allow_emaillist": {
                            "list": allow_emaillist or [],
                        },
                        "allow_departmentlist": {
                            "list": allow_departmentlist or [],
                        },
                        "allow_taglist": {"list": allow_taglist or []},
                    }
                )
            else:
                raise ValueError("Unsupported allow_type. Valid allow_types are 0, 1, 2 or 3")
        return self._post("exmail/group/create", data=data)

    def update_group(
        self,
        group_id,
        group_name=None,
        email_list=None,
        tag_list=None,
        department_list=None,
        group_list=None,
        allow_type=None,
        allow_emaillist=None,
        allow_departmentlist=None,
        allow_taglist=None,
    ):
        """
        更新邮件群组
        可以修改群组名称、群组成员、群组使用权限等。需要注意的是 Json 数组类型传空值将会清空其内容，不传则保持不变。

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95510

        参数含义见 create
        """
        data = {"groupid": group_id}
        if group_name is not None:
            data["groupname"] = group_name
        if email_list is not None:
            data["email_list"] = {"list": email_list}
        if tag_list is not None:
            data["tag_list"] = {"list": tag_list}
        if department_list is not None:
            data["department_list"] = {"list": department_list}
        if group_list is not None:
            data["group_list"] = {"list": group_list}
        if allow_type is not None:
            if allow_type not in {0, 1, 2, 3}:
                raise ValueError("Unsupported allow_type. Valid allow_types are 0, 1, 2 or 3")
            data["allow_type"] = allow_type
        if allow_emaillist is not None:
            data["allow_emaillist"] = {"list": allow_emaillist}
        if allow_departmentlist is not None:
            data["allow_departmentlist"] = {"list": allow_departmentlist}
        if allow_taglist is not None:
            data["allow_taglist"] = {"list": allow_taglist}
        return self._post("exmail/group/update", data=data)

    def delete_group(self, group_id):
        """
        删除邮件群组

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95510

        :param group_id: 必填，邮件群组 ID，邮箱格式。
        """
        return self._post("exmail/group/delete", data={"groupid": group_id})

    def get_group(self, group_id):
        """
        获取邮件群组详情

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95510

        :param group_id: 必填，邮件群组 ID，邮箱格式。
        """
        return self._get("exmail/group/get", params={"groupid": group_id})

    def search_group(self, fuzzy, group_id=None):
        """
        模糊搜索邮件群组，通过群组ID模糊搜索邮件群组。

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95510

        :param fuzzy: 必填，1 开启模糊搜索，0 获取全部邮件群组
        :param group_id: 选填，邮件群组ID，邮箱格式。
        """
        params = {"fuzzy": fuzzy}
        if group_id is not None:
            params["groupid"] = group_id
        return self._get("exmail/group/search", params=params, result_processor=itemgetter("groups"))

    def create_public_email(self, email, name, userid_list=None, department_list=None, tag_list=None):
        """
        创建业务邮箱

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param email: 必填，	业务邮箱地址。
        :param name：必填，业务邮箱名称，不多于 64 个字符或 32 个汉字，不得与其他业务邮箱重名。
        :param userid_list：选填，有权限使用业务邮箱的成员 UserID 列表。userid_list、department_list、taglist 不能同时为空。
        :param department_list：有权限使用业务邮箱的部门 ID 列表。
        :param tag_list：有权限使用业务邮箱的标签ID列表。
        """
        data = {"email": email, "name": name}
        if userid_list:
            data["userid_list"] = {"list": userid_list}
        if department_list:
            data["department_list"] = {"list": department_list}
        if tag_list:
            data["tag_list"] = {"list": tag_list}
        return self._post("exmail/publicmail/create", data=data, result_processor=itemgetter("id"))

    def update_public_email(self, email_id, name=None, userid_list=None, department_list=None, tag_list=None):
        """
        更新业务邮箱
        该接口用于更新业务邮箱，支持更新名称、使用权限。需要注意的是Json数组类型传空值将会清空其内容，不传则保持不变，传空为清空。

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param email_id: 必填，业务邮箱 ID。
        :param name：必填，业务邮箱名称，不多于 64 个字符或 32 个汉字，不得与其他业务邮箱重名。
        :param userid_list：选填，有权限使用业务邮箱的成员 UserID 列表。userid_list、department_list、taglist 不能同时为空。
        :param department_list：有权限使用业务邮箱的部门 ID 列表。
        :param tag_list：有权限使用业务邮箱的标签ID列表
        """
        data = {"id": email_id}
        if name is not None:
            data["name"] = name
        if userid_list is not None:
            data["userid_list"] = {"list": userid_list}
        if department_list is not None:
            data["department_list"] = {"list": department_list}
        if tag_list is not None:
            data["tag_list"] = {"list": tag_list}
        return self._post(
            "exmail/publicmail/update",
            data=data,
        )

    def delete_public_email(self, email_id):
        """
        删除业务邮箱

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param email_id: 必填，业务邮箱 ID。
        """
        return self._post(
            "exmail/publicmail/delete",
            data={"id": email_id},
        )

    def get_public_email(self, email_id):
        """
        获取业务邮箱详情

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param email_id: 必填，业务邮箱 ID。
        """
        return self.batch_get_public_email([email_id])[0]

    def batch_get_public_email(self, email_ids):
        """
        获取业务邮箱详情

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param email_ids: 必填，业务邮箱 ID 列表。
        """
        return self._post("exmail/publicmail/get", data={"id_list": email_ids}, result_processor=itemgetter("list"))

    def search_public_email(self, fuzzy, email=None):
        """
        模糊搜索业务邮箱
        用于模糊搜索业务邮箱，可匹配邮箱名也可匹配邮箱地址。

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95511

        :param fuzzy: 必填，1 开启模糊搜索，0 获取全部业务邮箱
        :param email: 选填，邮件群组 ID，邮箱格式。
        """
        params = {"fuzzy": fuzzy}
        if email is not None:
            params["email"] = email
        return self._get("exmail/publicmail/search", params=params, result_processor=itemgetter("list"))

    def active_email(self, user_id=None, public_email_id=None):
        """
        启用邮箱

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95512

        :param user_id: 成员 UserID，userid 与 public_email_id 至少应该传一项，同时传则只操作 userid。不可禁用超管与企业创建人。
        :param public_email_id: 业务邮箱 ID，userid 与 public_email_id 至少应该传一项，同时传则只操作 userid。
        """
        data = {"type": 1}
        if user_id:
            data["userid"] = user_id
        elif public_email_id:
            data["publicemail_id"] = public_email_id
        else:
            raise ValueError("user_id and public_email_id must be a choice.")

        return self._post(
            "exmail/account/act_email",
            data=data,
        )

    def inactive_email(self, user_id=None, public_email_id=None):
        """
        禁用邮箱

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95512

        :param user_id: 成员 UserID，userid 与 public_email_id 至少应该传一项，同时传则只操作 userid。不可禁用超管与企业创建人。
        :param public_email_id: 业务邮箱 ID，userid 与 public_email_id 至少应该传一项，同时传则只操作 userid。
        """
        data = {"type": 2}
        if user_id:
            data["userid"] = user_id
        elif public_email_id:
            data["publicemail_id"] = public_email_id
        else:
            raise ValueError("user_id and public_email_id must be a choice.")

        return self._post(
            "exmail/account/act_email",
            data=data,
        )

    def get_user_option(self, user_id, types):
        """
        获取用户功能属性

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95513

        :param user_id: 用户 UserID。
        :param types: 想查询的功能设置属性类型列表
            - 1: 强制启用安全登录
            - 2: IMAP/SMTP服务
            - 3: POP/SMTP服务
            - 4: 是否启用安全登录
        """
        return self._post(
            "exmail/useroption/get",
            data={
                "userid": user_id,
                "type": types,
            },
            result_processor=lambda x: x["option"]["list"],
        )

    def update_user_option(self, user_id, option):
        """
        更改用户功能属性

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95513

        :param user_id: 用户 UserID。
        :param option: 功能属性设置。

        option 数据示例：
        >>>[
        >>>    {
        >>>      "type": 1,
        >>>      "value": "0"
        >>>    },
        >>>    {
        >>>      "type": 2,
        >>>      "value": "1"
        >>>    },
        >>>    {
        >>>      "type": 3,
        >>>      "value": "0"
        >>>    }
        >>>]
        type: 功能属性类型
            - 1: 强制启用安全登录
            - 2: IMAP/SMTP服务
            - 3: POP/SMTP服务
            - 4: 是否启用安全登录
        value: 值
            - 1 表示启用
            - 0 表示关闭
        """
        return self._post(
            "exmail/useroption/update",
            data={
                "userid": user_id,
                "option": {"list": option},
            },
        )

    def get_new_email_count(self, user_id):
        """
        获取用户新邮件数

        详情请参考
        https://developer.work.weixin.qq.com/document/path/95514

        :param user_id: 用户 UserID。
        """
        return self._post(
            "exmail/mail/get_newcount",
            data={
                "userid": user_id,
            },
            result_processor=itemgetter("count"),
        )



# -*- coding: utf-8 -*-
from operator import itemgetter

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatUser(BaseWeChatAPI):
    def get(self, user_id, lang="zh_CN"):
        """
        获取用户基本信息（包括UnionID机制）

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html#UinonId

        :param user_id: 普通用户的标识，对当前公众号唯一
        :param lang: 返回国家地区语言版本，zh_CN 简体，zh_TW 繁体，en 英语
        :return: 返回的 JSON 数据包

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    user = client.user.get('openid')

        """
        assert lang in (
            "zh_CN",
            "zh_TW",
            "en",
        ), "lang can only be one of \
            zh_CN, zh_TW, en language codes"
        return self._get("user/info", params={"openid": user_id, "lang": lang})

    def get_followers(self, first_user_id=None):
        """
        获取一页用户列表(当关注用户过多的情况下，这个接口只会返回一部分用户)

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Getting_a_User_List.html

        :param first_user_id: 可选。第一个拉取的 OPENID，不填默认从头开始拉取
        :return: 返回的 JSON 数据包

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    followers = client.user.get_followers()

        """
        params = {}
        if first_user_id:
            params["next_openid"] = first_user_id
        return self._get("user/get", params=params)

    def iter_followers(self, first_user_id=None):
        """
        获取所有的用户openid列表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Getting_a_User_List.html

        :return: 返回一个迭代器，可以用for进行循环，得到openid

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    for openid in client.user.iter_followers():
        >>>        print(openid)

        """
        while True:
            follower_data = self.get_followers(first_user_id)
            first_user_id = follower_data["next_openid"]
            # 微信有个bug(或者叫feature)，没有下一页，也返回next_openid这个字段
            # 所以要通过total_count和data的长度比较判断(比较麻烦，并且不稳定)
            # 或者获得结果前先判断data是否存在
            if "data" not in follower_data:
                return
            for openid in follower_data["data"]["openid"]:
                yield openid
            if not first_user_id:
                return

    def update_remark(self, user_id, remark):
        """
        设置用户备注名

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Configuring_user_notes.html

        :param user_id: 用户标识
        :param remark: 新的备注名，长度必须小于30字符
        :return: 返回的 JSON 数据包

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    client.user.update_remark('openid', 'Remark')

        """
        return self._post("user/info/updateremark", data={"openid": user_id, "remark": remark})

    def get_group_id(self, user_id):
        """
        ⚠️已废弃
        获取用户所在分组 ID

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param user_id: 用户 ID
        :return: 用户所在分组 ID

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            group_id = client.user.get_group_id('openid')

        """
        res = self._post(
            "groups/getid",
            data={"openid": user_id},
            result_processor=itemgetter("groupid"),
        )
        return res

    def get_batch(self, user_list):
        """
        批量获取用户基本信息
        开发者可通过该接口来批量获取用户基本信息。最多支持一次拉取100条。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/User_Management/Get_users_basic_information_UnionID.html#UinonId

        :param user_list: user_list，支持“使用示例”中两种输入格式
        :return: 用户信息的 list

        使用示例::

        >>>    from wechatpy import WeChatClient
        >>>
        >>>    client = WeChatClient('appid', 'secret')
        >>>    users = client.user.get_batch(['openid1', 'openid2'])
        >>>    users = client.user.get_batch([
        >>>      {'openid': 'openid1', 'lang': 'zh-CN'},
        >>>      {'openid': 'openid2', 'lang': 'en'},
        >>>    ])

        opendid: 必填，用户的标识，对当前公众号唯一
        lang: 非必填，国家地区语言版本，zh_CN 简体，zh_TW 繁体，en 英语，默认为zh-CN
        """
        if all((isinstance(x, str) for x in user_list)):
            user_list = [{"openid": oid} for oid in user_list]
        res = self._post(
            "user/info/batchget",
            data={"user_list": user_list},
            result_processor=itemgetter("user_info_list"),
        )
        return res

    def change_openid(self, from_appid, openid_list):
        """微信公众号主体变更迁移用户 openid

        详情请参考
        http://kf.qq.com/faq/170221aUnmmU170221eUZJNf.html

        :param from_appid: 原公众号的 appid
        :param openid_list: 需要转换的openid，这些必须是旧账号目前关注的才行，否则会出错；一次最多100个
        :return: 转换后的 openid 信息列表
        """
        return self._post(
            "changeopenid",
            data={"from_appid": from_appid, "openid_list": openid_list},
            result_processor=itemgetter("result_list"),
        )



# -*- coding: utf-8 -*-
from typing import List

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatExternalContactGroupChat(BaseWeChatAPI):
    """
    https://work.weixin.qq.com/api/doc#90000/90135/90221
    """

    def list(
        self,
        status_filter: int = 0,
        owner_userid_list: List = None,
        cursor: str = None,
        limit: int = 100,
    ):
        """
        该接口用于获取配置过客户群管理的客户群列表。
        https://work.weixin.qq.com/api/doc/90000/90135/92120
        :return: 返回的 JSON 数据包
        :param status_filter: 客户群跟进状态过滤。[0 - 所有列表(即不过滤)
                                                1 - 离职待继承
                                                2 - 离职继承中
                                                3 - 离职继承完成

                                                默认为0]
        :param owner_userid_list: 根据群主id过滤。如果不填，表示获取应用可见范围内全部群主的数据（但是不建议这么用，如果可见范围人数超过1000人，为了防止数据包过大，会报错 81017）
        :param cursor: 用于分页查询的游标，字符串类型，由上一次调用返回，首次调用不填
        :param limit: 分页，预期请求的数据量，取值范围 1 ~ 1000， 默认100
        :return:
        """
        data = optionaldict(
            status_filter=status_filter,
            cursor=cursor,
            limit=limit,
        )
        if owner_userid_list:
            data["owner_filter"] = {"userid_list": owner_userid_list}
        return self._post("externalcontact/groupchat/list", data=data)

    def list_all(self, status_filter: int = 0, owner_userid_list: List = None, limit: int = 100) -> List:
        """
        该接口用于获取配置过客户群管理的所有客户群列表，自动走完所有分页
        """
        chat_list = []
        cursor = None
        while True:
            result = self.list(status_filter, owner_userid_list, cursor, limit)
            if result["errcode"] == 0:
                chat_list.extend(result["group_chat_list"])
            if result.get("next_cursor"):
                cursor = result["next_cursor"]
            else:
                break
        return chat_list

    def get(self, chat_id: str):
        """
        通过客户群ID，获取详情。包括群名、群成员列表、群成员入群时间、入群方式。（客户群是由具有客户群使用权限的成员创建的外部群）
        需注意的是，如果发生群信息变动，会立即收到群变更事件，但是部分信息是异步处理，可能需要等一段时间调此接口才能得到最新结果
        PS: 接口命名为get，调用却是POST，无语
        https://work.weixin.qq.com/api/doc/90000/90135/92122
        :param chat_id: 客户群ID
        :return: 返回的 JSON 数据包
        """
        return self._post("externalcontact/groupchat/get", data={"chat_id": chat_id})

    def statistic(
        self,
        day_begin_time: int,
        day_end_time: int = None,
        owner_userid_list: List = None,
        order_by: int = 1,
        order_asc: int = 0,
        offset: int = 0,
        limit: int = 500,
    ):
        """
        获取指定日期的统计数据。注意，企业微信仅存储180天的数据。
        :param day_begin_time: 起始日期的时间戳，填当天的0时0分0秒（否则系统自动处理为当天的0分0秒）。取值范围：昨天至前180天。
        :param day_end_time: 结束日期的时间戳，填当天的0时0分0秒（否则系统自动处理为当天的0分0秒）。取值范围：昨天至前180天。
                            如果不填，默认同 day_begin_time（即默认取一天的数据）
        :param owner_userid_list: 群主过滤，如果不填，表示获取全部群主的数据
        :param order_by:    排序方式。默认为1
                            [1 - 新增群的数量
                            2 - 群总数
                            3 - 新增群人数
                            4 - 群总人数]
        :param order_asc: 是否升序。0-否；1-是。默认降序,即0
        :param offset: 分页，偏移量, 默认为0
        :param limit: 分页，预期请求的数据量，默认为500，取值范围 1 ~ 1000
        :return: 返回的 JSON 数据包
        """
        if not day_end_time:
            day_end_time = day_begin_time
        data = optionaldict(
            day_begin_time=day_begin_time,
            day_end_time=day_end_time,
            order_by=order_by,
            order_asc=order_asc,
            offset=offset,
            limit=limit,
        )
        if owner_userid_list:
            data["owner_filter"] = {"userid_list": owner_userid_list}
        return self._post("externalcontact/groupchat/statistic", data=data)
